/*====================================================================
 
 Objectweb Browser Framework
 Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
 Contact: openccm@objectweb.org
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 USA
 
 Initial developer(s): Romain Rouvoy.
 Contributor(s): ______________________________________.
 
 ---------------------------------------------------------------------
 $Id: NameProvider.java,v 1.1 2004/04/14 09:43:14 rouvoy Exp $
 ====================================================================*/

package org.objectweb.util.browser.plugins.fractal.context;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.util.Fractal;

/**
 * Name Provider for Fractal.
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 * @version 0.1
 */
public class NameProvider 
{
    /** Default instance of the NameProvider. */
    protected static final NameProvider singleton = new NameProvider();
    
    /**
     * Provides the instance of the NameProvider.
     * @return default instance of the name provider.
     */
    public static NameProvider singleton() {
        return singleton;
    }
    
    /**
     * Provides the reference to the component associated to this object.
     * @param obj the base object reference.
     * @return the associated component.
     */
    public Component getComponent(Object obj) {
        try {
            return (Component)obj;
        } catch(ClassCastException ex) { }
        return ((Interface)obj).getFcItfOwner(); 
    }
    
    /**
     * Provides the name of the object prefixed by the component name.
     * @param obj the object to identify.
     * @return the associated name.
     */
    public String getPrefixedName(Object obj) {
        return getName(getComponent(obj))+"."+getName(obj);
    }
    
    /**
     * Provides the name of an object.
     * @param obj the object to identify.
     * @return the associated name ('unknown' if no name).
     */
    public String getName(Object obj) {
        try {
            return getName((Component)obj);
        } catch(ClassCastException ex) { }
        try {
            return getName((Interface)obj);
        } catch(ClassCastException ex) { }
        return "unknown" ;
    }
    
    public String getName(InterfaceType itf) {
        return itf.getFcItfName();
    }
    
    /**
     * Provides the name of an interface.
     * @param itf the interface to identify.
     * @return the name of the interface.
     */
    public String getName(Interface itf) {
        return itf.getFcItfName();
    }
    
    /**
     * Provides the name of a component.
     * @param cpt the component to identify.
     * @return the name of the component (the name of the associated 
     * interface if no NameController).
     */
    public String getName(Component cpt) {
        try {
            return Fractal.getNameController(cpt).getFcName();
        } catch (NoSuchInterfaceException e) {
            return getName((Interface)cpt);
        }
    }
}
