/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.plugin.java.map;

import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.Wrapper;
import org.objectweb.util.browser.core.naming.DefaultEntry;
import org.objectweb.util.browser.core.naming.DefaultName;

/**
 * This class allows to browse a <code>java.util.Map</code> object.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.1
 */
public class MapContext 
  implements Wrapper, Context 
{

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    /** The reference of the map to browse. */
    protected java.util.Map map_;

    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods for Wrapper interface.
    //
    //==================================================================

    /**
     * Sets the wrapped object.
     */
    public void setWrapped(Object object) 
    {
        map_ = (java.util.Map)object;
    }

    /**
     * Gets the wrapped object.
     */
    public Object getWrapped() 
    {
        return map_;
    }

    //==================================================================
    //
    // Public methods for Context interface.
    //
    //==================================================================

    /**
     * Returns an array of Entry representing the mappings contained in this map.
     * @return The entries of the context.
     */
    public Entry[] getEntries() 
    {
        // Creates the array of entries.
        Entry[] entries = new Entry[map_.size()];
        
        // Populates the array with the mappings contained in the map.
        Object[] keys = map_.keySet().toArray();
        Object[] elements = map_.values().toArray();
        for (int i = 0; i < entries.length; i++) 
        {
            entries[i] =  
              new DefaultEntry(elements[i], new DefaultName(keys[i].toString()), this);
        }
        return entries;
    }

}