/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.api;

import javax.swing.JPanel;

/**
 * <p>
 * Contains all the methods called by the console on the viewPanel.<br>
 * In order to define a panel, you can have a look to the corresponding
 * <a href="../../../../../../resources/browser_dtd.html#panel">DTD element</a> or to 
 * an <a href="package-summary.html#xml">XML example.</a>
 * </p>
 *
 * <p>
 * Example:
 * <pre>
 *      public void 
 *      selected(TreeView treeView) 
 *      {
 *          String[] titre = {"Keys","Values"};
 *          Properties properties = (Properties)treeView.getSelectedObject();
 *          Enumeration keys = properties.keys();
 *          Enumeration elements = properties.elements();
 *          String[][] contenu = new String[properties.size()][2];
 *          for (int i = 0; i &lt; properties.size(); i++) {
 *              contenu[i] = new String[]{(String) keys.nextElement(),(String)elements.nextElement()}; 
 *          }
 *          panel_.add(new JScrollPane(new JTable(contenu,titre)));
 *      }
 *
 *      public JPanel 
 *      getPanel() 
 *      {
 *          return panel_;
 *      }
 *
 *      public void 
 *      unselected(TreeView treeView) 
 *      {
 *          // Nothing to do
 *      }
 * </pre>
 * This panel creates a JTable containing all the property registered in 
 * a <code>java.util.Properties<code> object.
 * This JTable is divided into two parts: A "key" column containing the 
 * key values and a "value" column containing the values of the property. 
 * </p>
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.2
 */
public interface Panel 
         extends View
{

    /**
     * Invokes just after instanciation.
     * 
     * @param treeview A view on the tree.
     */
    public void selected(TreeView treeview);

    /**
     * Provides the panel to display.
     * 
     * @return The panel to display.
     */
    public JPanel getPanel();

    /**
     * Invokes just before removing.
     * 
     * @param treeview A view on the tree.
     */
    public void unselected(TreeView treeview);

}