/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.api;

/** The console's imports  */
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.core.common.ExtendedBoolean;

/**
 * Interface which allow to try to load a new property concerning an object (icon, panel, menu, wrapper)
 * It looks for the id in its context which contains all the node property.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public interface BrowserProperty {

    /**
     * Fixes the context property to use to constructs the wrappers
     * @param properties The context property to use
     */
    public void setContextProperty(ContextProperty properties);

    /**
     * Provides the TypeSystem to use in order to find appropriate key.
     * @return The TypeSystem to use.
     */
    public TypeSystem getTypeSystem();

    /**
     * Provides the entry to add into the icon property context or null.
     * @param id The research key
     * @return The entry to add or null
     */
    public Entry getIcon(String id, Role role, ExtendedBoolean nodeFound);

    /**
     * Provides the entry to add into the panel property context or null.
     * @param id The research key
     * @return The entry to add or null
     */
    public Entry getPanel(String id, Role role, ExtendedBoolean nodeFound);

    /**
     * Provides the entry to add into the menu property context or null.
     * @param id The research key (a class name)
     * @param role The associated role
     * @param nodeFound Indicates if a node has been found or not.
     * @return The entry to add or null
     */
    public Entry getMenu(String id, Role role, ExtendedBoolean nodeFound);

    /**
     * Provides the entry to add into the wrapper property context or null.
     * @param id The research key
     * @return The entry to add or null
     */
    public Entry getWrapper(String id, Role role, ExtendedBoolean nodeFound);
    
    /**
     * Provides the entry to add into the dropaction property context or null.
     * @param id The research key
     * @return The entry to add or null
     */
    public Entry getDropAction(String id, Role role, ExtendedBoolean nodeFound);

    /**
     * Provides the entry to add into the info property context or null.
     * @param id The research key
     * @return The associated entry or null
     */
    public Entry getInfo(String id, Role role, ExtendedBoolean nodeFound);

    /**
     * Provides the initial context factory for the given role.
     */
    public RootFactory getRoot(Role role);
  
    /**
     * Provides the inherited roles of the current role. If no inherited node is defined, an empty array is return.
     * @param role The role. 
     * @return An array containing the inherited roles for the current role.
     */
    public Role[] getInheritedRoles();  
}
