/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.naming;

/** To use the Name interface. */
import org.objectweb.util.browser.core.api.Name;

/**
 * Default basic implementation of the Name interface.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @version 0.1
 */
public class DefaultName extends AbstractName {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The identifier. */
    private String id_;

    /** The subname. */
    private Name subname_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Constructs a DefaultName with no id and no subname.
     *
     * This constructor sets the id and the subname to the null value.
     *
     * @postcondition
     *           getOWId() == null
     *  and getOWSubName() == null;
     */
    public DefaultName() {
        this(null, null);
    }

    /**
     * Constructs a DefaultName with an id and no subname.
     *
     * This constructor sets the id to the given id
     * and the subname to the null value.
     *
     * @param id The identifier.
     *
     * @postcondition
     *           getOWId() == id
     *  and getOWSubName() == null;
     */
    public DefaultName(String id) {
        this(id, null);
    }

    /**
     * Constructs a DefaultName with an id and a subname.
     *
     * This constructor sets the id and the subname
     * to respectively the given id and subname.
     *
     * @param id The identifier.
     * @param subname The subname.
     *
     * @postcondition
     *           getOWId() == id
     *  and getOWSubName() == subname;
     */
    public DefaultName(String id, Name subname) {
        // Inits the internal state.
        id_ = id;
        subname_ = subname;
    }

    // ==================================================================
    //
    // No internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for interface Name.
    //
    // ==================================================================

    /**
     * Gets the id of the target name.
     *
     * This method does not update the id of the target name.
     *
     * @return The id of the target name.
     *
     * @postcondition return == getOWId();
     */
    public String getOWId() {
        return id_;
    }

    /**
     * Gets the subname of the target name.
     *
     * This method does not update the subname of the target name.
     *
     * @retrn the subname of the target name.
     *
     * @postcondition return == getOWSubName();
     */
    public Name getOWSubName() {
        return subname_;
    }

}
