/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jrme Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.panel;

/** To use the PanelFactory interface */
import java.util.Iterator;
import java.util.Vector;

import org.objectweb.util.browser.api.Panel;
import org.objectweb.util.browser.core.api.PanelFactory;

/**
 * Basic implementation for Panel factories.
 *
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 *
 */
public class DefaultPanelFactory 
    implements PanelFactory {

    /** The Associated class */
    protected Class class_ = null;

    /** Indicates if this panel has to inherit panel from parents at the java type inheritance level. */
    protected boolean inheritTypePanel_ = false;
    
    protected Vector subPanels_ = new Vector();

    protected Panel createPanel(Object object){
        if (class_ != null) {
            try {
                return (Panel) class_.newInstance();
            } catch (java.lang.InstantiationException e) {
                System.out.println(class_.getName() + " : Instanciation exception");
            } catch (java.lang.IllegalAccessException e) {
                System.out.println(class_.getName() + " : Illegal access exception");
            }
        }
        return null;        
    }

    /**
     * Return an JPanel associated to the param's object.
     *
     * @return The associated panel.
     */
    public Panel newPanel(Object object) {
        if (subPanels_.size()>0){
            DefaultCompositePanel compositePanel = new DefaultCompositePanel();
            compositePanel.add(createPanel(object));
            Iterator it = subPanels_.iterator();
            while(it.hasNext()){
                compositePanel.add(((PanelFactory)it.next()).newPanel(object));
            }
            return compositePanel;
        } else {
            return createPanel(object);
        }
    }

    /**
     * Fixes the panel class to display
     * 
     * @param theClass The class to instantiate
     */
    public void setClassName(Class theClass) {
        class_ = theClass;
    }

    /**
     * Fixes the inheritTypePanel value.
     * @param inheritTypePanel true if this panel has to inherit panel from parents at the java type inheritance level 
     */
    public void setInheritTypePanel(boolean inheritTypePanel){
        inheritTypePanel_ = inheritTypePanel;
    }
    
    /**
     * Indicates if this panel has to inherit panel from parents at the java type inheritance level.
     */
    public boolean inheritTypePanel(){
        return inheritTypePanel_;
    }

    /**
     * Addes a PanelFactory
     * @param panelFactory The PanelFactory to add.
     */
    public void addInheritPanelFactory(PanelFactory panelFactory){
        subPanels_.add(panelFactory);
    }
}
