/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2003 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.xmlparser;

/** To use the generated class */
import org.objectweb.util.browser.browserConfig.Browser;
import org.objectweb.util.browser.browserConfig.beans.BrowserBeanImpl;
import org.objectweb.util.browser.browserConfig.beans.BrowserBean;
import org.objectweb.util.browser.browserConfig.Code;
import org.objectweb.util.browser.browserConfig.CodeImpl;
import org.objectweb.util.browser.browserConfig.DropAction;
import org.objectweb.util.browser.browserConfig.IconImpl;
import org.objectweb.util.browser.browserConfig.Include;
import org.objectweb.util.browser.browserConfig.Info;
import org.objectweb.util.browser.browserConfig.Mnemonic;
import org.objectweb.util.browser.browserConfig.Node;
import org.objectweb.util.browser.browserConfig.Icon;
import org.objectweb.util.browser.browserConfig.NodeImpl;
import org.objectweb.util.browser.browserConfig.Table;
import org.objectweb.util.browser.browserConfig.TypeSystem;
import org.objectweb.util.browser.browserConfig.Wrapper;
import org.objectweb.util.browser.browserConfig.beans.PanelBean;
import org.objectweb.util.browser.browserConfig.Panel;
import org.objectweb.util.browser.browserConfig.Menu;
import org.objectweb.util.browser.browserConfig.Item;
import org.objectweb.util.browser.browserConfig.Jar;
import org.objectweb.util.browser.browserConfig.Zip;
import org.objectweb.util.browser.browserConfig.Root;
import org.objectweb.util.browser.browserConfig.Accelerator;

/** The console's imports */
import org.objectweb.util.browser.core.naming.DefaultEntry;
import org.objectweb.util.browser.core.naming.DefaultName;
import org.objectweb.util.browser.core.naming.DefaultContextContainer;
import org.objectweb.util.browser.core.naming.DefaultContextFactory;
import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.IconProvider;
import org.objectweb.util.browser.core.api.BrowserProperty;
import org.objectweb.util.browser.core.api.ContextContainer;
import org.objectweb.util.browser.core.api.ContextProperty;
import org.objectweb.util.browser.core.api.Decoder;
import org.objectweb.util.browser.core.api.DropActionFactory;
import org.objectweb.util.browser.core.api.FileIconProvider;
import org.objectweb.util.browser.core.api.InfoFactory;
import org.objectweb.util.browser.core.api.MenuFactoryConfiguration;
import org.objectweb.util.browser.core.api.PanelFactory;
import org.objectweb.util.browser.core.api.Role;
import org.objectweb.util.browser.core.api.RoleManagement;
import org.objectweb.util.browser.core.api.SpecificContextFactory;
import org.objectweb.util.browser.core.api.TableFactory;
import org.objectweb.util.browser.core.api.TypeSystemHandler;
import org.objectweb.util.browser.core.api.ViewFactory;
import org.objectweb.util.browser.core.api.RootFactory;
import org.objectweb.util.browser.core.common.ClassResolver;
import org.objectweb.util.browser.core.common.DefaultRole;
import org.objectweb.util.browser.core.common.DefaultTypeSystemHandler;
import org.objectweb.util.browser.core.common.DynamicTree;
import org.objectweb.util.browser.core.common.ExtendedBoolean;
import org.objectweb.util.browser.core.common.JavaTypeSystem;
import org.objectweb.util.browser.core.common.DefaultRootFactory;
import org.objectweb.util.browser.core.dnd.DefaultDropActionFactory;
import org.objectweb.util.browser.core.icon.DefaultFileIconProvider;
import org.objectweb.util.browser.core.info.DefaultInfoFactory;
import org.objectweb.util.browser.core.panel.DefaultCompositePanelFactory;
import org.objectweb.util.browser.core.panel.DefaultPanelFactory;
import org.objectweb.util.browser.core.panel.DefaultTableFactory;
import org.objectweb.util.browser.core.popup.DefaultMenuFactory;
import org.objectweb.util.browser.core.popup.ItemProperty;

/** The java API's imports */
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.KeyStroke;

/**
 * Parses a Java object which represents an XML Configuration.
 * A configuration for a node is firstly defined for a role and then for a kind of object. 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class BrowserXMLParser 
    implements XMLParser, BrowserProperty, RoleManagement {
	
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The context which contains the XML properties. 
     *  The roleConfig_ context contains the inherited role of a role and 
     */
    protected ContextContainer role_context_, roleConfig_, inheritedRolesMap_;

    /**  The roleMap_ context stores the Role for a given role id. */
    protected Map roleMap_;

    /** The rootMap_ contains the definition of the root context. */
    protected Map rootMap_;

    /** The context property to use */
    protected ContextProperty contextProperty_;

    /** The decoder to use */
    protected Decoder decoder_;

    /** The TypeSystemHandler */
    protected TypeSystemHandler typeSystemHandler_;

    /** A reference to the tree */
    protected DynamicTree tree_;
    
    /** The current role */
    protected Role default_role_;
    
    /** The list of current roles */
    protected Role[] current_roles_, current_roles_inheritance_;
    
    protected boolean sameRoleList_ = false;

    /** The directory in which the zip file will be extracted. */    
    protected File extractedFolder_ = null;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default constructor
     */
    public BrowserXMLParser(DynamicTree tree) {
        role_context_ = new DefaultContextContainer();
        roleConfig_ = new DefaultContextContainer();
        roleMap_ = new HashMap();
        rootMap_ = new HashMap();
        inheritedRolesMap_ = new DefaultContextContainer();
        typeSystemHandler_ = new DefaultTypeSystemHandler();
        tree_ = tree;
        default_role_ = new DefaultRole(RoleManagement.DEFAULT_ROLE);
        current_roles_ = new Role[]{default_role_};
        
        // In order to remove the temporary folder before stopping the program
        System.runFinalizersOnExit(true);
        
        // Ceates a temp directory
        String tmpDir = System.getProperty("java.io.tmpdir");
        if ((!tmpDir.endsWith("/")) && (!tmpDir.endsWith("\\")))
            tmpDir = tmpDir + "/";
        extractedFolder_ = new File(tmpDir + "browser" + System.currentTimeMillis());
        extractedFolder_.mkdir();
        extractedFolder_.deleteOnExit();        
        
    }   

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Creates a node entry for java.lang.Object for the given role.
     * @param role The role
     */
    protected Node getDefaultObjectConfig(){
        // Default config for java.lang.Object
        Node objectNode = new NodeImpl();
        objectNode.setTypeName("java.lang.Object");
        Icon objectIcon = new IconImpl();
        Code iconCode = new CodeImpl();
        iconCode.setValue("org.objectweb.util.browser.core.icon.ObjectIconFactory");
        objectIcon.setCode(iconCode);
        objectNode.setIcon(objectIcon);
        //mergeConfig(objectNode.getTypeSystem() + "." + objectNode.getTypeName(), objectNode, role);
        return objectNode;
    }
    
    /**
     * Add an Include element into the context
     *
     * @param include The Include element to interpret
     */
    protected void addInclude(Include include) {
        InputStream inputStream = null;
        // Tries to load the ressource from the ClassLoader
        inputStream = ClassResolver.getResourceAsStream(include.getUrl());
        if(inputStream==null){
            System.err.println(include.getUrl() + ": resource not found in the ClassLoader");        
            try {
                URL url = new URL(include.getUrl());
                buildBrowser(url.openStream());
            } catch (java.net.MalformedURLException e) {
                if (include.getUrl() != null && !include.getUrl().equals(""))
                    System.err.println(include.getUrl() + ": Malformed URL !");
                else
                    System.err.println("null URL exists in the XML file !");
            } catch (java.io.FileNotFoundException e) {
                System.err.println(include.getUrl() + " : File not found !");
            } catch (java.io.IOException e) {
                System.err.println(e.getMessage());
            }
        } else {
            buildBrowser(inputStream);
        }
    }

    /** 
     * Provide the class corresponding to the code element
     * @param code The code element
     * @return The corresponding class 
     */
    protected Class getClass(Code code) {
        // only Java is supported for the moment !!!
        if(code!=null){
            String className = code.getValue();
            try{
                return ClassResolver.resolve(className);
            }catch(ClassNotFoundException e){
                System.err.println("[BrowserXMLParser.getClass()] " + e + ": ClassNotFoundException");
                return null;   
            }
        }
        return null;
    }

    /**
     * Add a TypeSystem element into the TypeSystemHandler
     * 
     * @param typeSystem The TypeSystem to add
     */
    protected void addTypeSystem(TypeSystem typeSystem) {
        if(typeSystem!=null){
            Class c = getClass(typeSystem.getCode());
            if(org.objectweb.util.browser.core.api.TypeSystem.class.isAssignableFrom(c)){
				try {
					typeSystemHandler_.addTypeSystem((org.objectweb.util.browser.core.api.TypeSystem)c.newInstance());
				} catch (InstantiationException e) {
                    System.err.println("[BrowserXMLParser.addTypeSystem()]");
					e.printStackTrace();
				} catch (IllegalAccessException e) {
                    System.err.println("[BrowserXMLParser.addTypeSystem()]");
					e.printStackTrace();
				}
            }
        }
    }

    /**
     * Creates an Entry.
     *
     * @param value The value of the entry to create.
     * @param id The id of the name of the entry to create.
     *
     * @return A new Entry.
     */
    protected Entry createEntry(String id, Object value) {
        return new DefaultEntry(value, new DefaultName(id), null);
    }

    /**
     * Loads the URL of the jar into the class loader
     * @param jarURL The URL to load.
     */
    protected void loadJarURL(URL[] jarURL){
        if(jarURL!=null)
            ClassResolver.addContext(jarURL);
    }

    /**
     * Loads the jar into the Browser CLassLoader
     * @param jarList A list of jar to load
     */
    protected void loadJar(List jarList){
        List urlList = new Vector();
        for (Iterator i = jarList.iterator(); i.hasNext();) {
            Object object = i.next();
            if(object!=null && Jar.class.isAssignableFrom(object.getClass())){
                Jar jar = (Jar)object; 
                String jarFile = jar.getUrl();
                try {
                    urlList.add(new URL(jarFile));
                } catch (java.net.MalformedURLException e) {
                    System.err.println(jarFile + " : Malformed URL !");
                }
            }
        }
        if (!urlList.isEmpty()) {
            loadJarURL((URL[]) urlList.toArray(new URL[0]));
        }
    }

    /**
     * Removes the given file and recursively removes if the target file is a directory.
     */
    public void remove(File file){
        if(file!=null)  {
            if(file.isDirectory()){
                File[] children = file.listFiles();
                for(int i = 0 ; i < children.length ; i++) 
                    remove(children[i]);
            }
            file.delete();
        } 
    }

    /**
     * Writes a file on the given input stream.
     * Note that the stream are closed when leaving the this method.
     */
    protected void writeFile(InputStream input, OutputStream output) throws IOException {
        int bufferSize = 1024;
        byte[] buffer = new byte[bufferSize];
        int length=0;
        while((length = input.read(buffer))>0){
            output.write(buffer, 0, length);
        }
        output.close();
        input.close(); 
    }

    /**
     * Extracts a ZipEntry into the tmp dir.
     * @param file The zip file.
     * @param entry The entry to extract.
     * @throws IOException Throws when an IOException occured.
     */
    protected void extractEntry(ZipFile file, ZipEntry entry, String fileName) throws IOException {
        writeFile(file.getInputStream(entry), new FileOutputStream(fileName));
    }

    /*
     * It seems tere is a problem with the methods "mkdirs()" and "deleteOnExit()" when they are working together.
     * Indeed, the mkdirs method recursively creates the directory contained in this File definition and the deleteOnExit method
     * removes those files when the virtual machine terminates, but when more than one directory is defined in the File, the first one
     * is not removed by the deleteOnExit calling.
     * It is the only problem that prevents this class to remove the "finalize()" method. 
     */
    /**
     * Creates the given directory.
     * @param path The path of the disrectory to create
     */
    protected void createDirectory(String path){
        File directory = new File(path);
        if(!directory.exists()){
            // the mkdirs method (used with deleteOnExit) doesn't remove the first directory of a directories path.
            directory.deleteOnExit();  
            directory.mkdirs();
        }
    }

    /**
     * Extracts a Zip archive
     */
    protected void extractFile(File file, Vector javaArchives) throws IOException {
        ZipFile zipFile = null;
        try {
            zipFile = new ZipFile(file.toString());
        } catch (ZipException e) {
            // Nothing to do
        } finally {
            file.deleteOnExit();
        }
        if(zipFile!=null){
            for(Enumeration enum = zipFile.entries();enum.hasMoreElements();){
                ZipEntry zipEntry = (ZipEntry)enum.nextElement();
                if(zipEntry.isDirectory()){
                    createDirectory(extractedFolder_.getCanonicalPath() + "/" + zipEntry.getName());
                } else {
                    // Creates the directories
                    int indice = zipEntry.getName().lastIndexOf('/');
                    if(indice!=-1){
                        createDirectory(extractedFolder_.getCanonicalPath() + "/" + zipEntry.getName().substring(0,indice));
                    }
                    // Computes the name of the file
                    String fileName = extractedFolder_.getCanonicalPath() + "/" + zipEntry.getName();
                    // Extracts the entry
                    try {
                        extractEntry(zipFile, zipEntry, fileName);
                    } catch (IOException e) {
                        System.err.println("IOException: " + e.getMessage());
                        e.printStackTrace();
                    }
                    if(zipEntry.getName().toLowerCase().endsWith(".jar")) {
                        // Adds the java archive into the vector containing the Java archive to load
                        File jarFile = new File(fileName);
                        jarFile.deleteOnExit();
                        try {
                            javaArchives.add(jarFile.toURL());
                        } catch (IOException e) {
                            System.err.println("IOException: " + e.getMessage());
                            e.printStackTrace();
                        }
                    } else {
                        extractFile(new File(fileName), javaArchives);
                    }
                }
            }
        }
    }

    /**
     * Loads the contained jar archives of the given zip element.
     * @param zip A zip archive containing the jar archives to load.
     */
    protected void loadZip(URL zipURL) throws IOException {
        if(zipURL!=null){
            Vector javaArchives = new Vector();

            // Copies the zip file into the java temp directory
            String fileName = extractedFolder_.getCanonicalPath() + "/temp.zip";
            writeFile(zipURL.openStream(), new FileOutputStream(fileName));

            // Extracts the zip file     
            File zipFile = new File(fileName);
            zipFile.deleteOnExit();                               
            extractFile(zipFile, javaArchives);
            
            // Loads the jar contained in the given zip file
            loadJarURL((URL[])javaArchives.toArray(new URL[0]));
        }
    }

    /**
     * Unzips all the zip archives of the given list and for each archive, loads the contained jar archives.
     * @param zipList A list of zip archives containing the jar archives to load.
     */
    protected void loadZips(List zipList){
        if(zipList!=null){
            for(Iterator i = zipList.iterator() ; i.hasNext() ; ){
                Zip zip = (Zip)i.next();
                try {
                    loadZip(new URL(zip.getUrl()));
                } catch (MalformedURLException e) {
                    System.err.println("MalformedURLException: " + e.getMessage());
                } catch (IOException e) {
                    System.err.println("IOException: " + e.getMessage());
                }
            }
        }
    }

    /**
     * Looks for a role config for the given role.
     * @param role The key of the search.
     * @return The associated role config (or null)
     */
    protected ContextContainer getRoleContext(Role role){
        if(role!=null)
            return (ContextContainer)role_context_.getLocalValue(role.getId());
        return null;
    }
    
    /**
     * Looks for the node with the given id for the given role. 
     * If no node is found for the given role and if a node exists for the default_role, this one is returned
     * @param id The target of the node.
     * @param role The role 
     * @param lookForInDefaultRole If no node is found for the given role and if a node exists for the default_role, this one is returned
     * @return The associated node (or null)
     */
    protected Node getNode(String id, Role role, boolean lookForInDefaultRole) {
        // The lookForInDefaultRole allows us to load a configuration for different roles.
        ContextContainer roleContext = getRoleContext(role);
        if(roleContext!=null) {
            Node node = (Node) roleContext.getLocalValue(id);
            if(node==null && !role.getId().equals(default_role_.getId()) && lookForInDefaultRole)
                return getNode(id,default_role_,lookForInDefaultRole);
            return node;
        } else if (!role.getId().equals(default_role_.getId()) && lookForInDefaultRole){
            return getNode(id,default_role_,lookForInDefaultRole);
        }
        return null;
    }

    /**
     * Merges two similar nodes if a node exists in the property context.Else adds the node into the property config
     * If an element of the node is already define in the property context's node, this one is replaced by this which is contained in the given node.
     * @param id Id of the node to merge.
     * @param nodeToMerge The node to merge with the one contains in the property context.
     * @param role The role for which the node is defined.
     */
    protected void mergeConfig(String id, Node nodeToMerge, Role role){
        ContextContainer roleContext = getRoleContext(role);
        if(roleContext != null) {
            Node node = getNode(id,role,false);
            if(node!=null){
                // Merge the wrapper config
                Wrapper wrapper = nodeToMerge.getWrapper();
                if (wrapper != null) 
                    node.setWrapper(wrapper);
                // Merge the icon config
                Icon icon = nodeToMerge.getIcon();
                if (icon != null)
                    node.setIcon(icon);
                // Merge the panel config
                Panel panel = nodeToMerge.getPanel();
                if (panel != null)
                    node.setPanel(panel);
                // Merge the info config
                Info info = nodeToMerge.getInfo();
                if (info != null)
                    node.setInfo(info);
                // Merge the menu config
                Menu menuToMerge = nodeToMerge.getMenu();
                if (menuToMerge != null) {
                    Menu menu = node.getMenu();
                    if(menu==null)
                        node.setMenu(menuToMerge);
                    else{
                        if (menuToMerge.getInheritTreeMenu().equals("false")) 
                            menu.setInheritTreeMenu("false");
                        if (menuToMerge.getInheritTypeMenu().equals("false"))
                            menu.setInheritTypeMenu("false");
                        List itemsToMerge = menuToMerge.getItemList();    
                        if (itemsToMerge!=null && itemsToMerge.size()>0){
                            Iterator it = itemsToMerge.iterator();
                            while (it.hasNext()) {
                                Item item = (Item) it.next();
                                menu.addItem(item);
                            }
                        }
                    }
                }
                // Merge the drag&drop action config
                DropAction drop_action = nodeToMerge.getDropAction();
                if (drop_action != null)
                    node.setDropAction(drop_action);
            } else {
                // Adds the node into the role context
                roleContext.addEntry(id, nodeToMerge);
            }           
        } else {
            // Saves the role map and the node into the context
            ContextContainer newRoleContext = new DefaultContextContainer();
            // Adding a default configuration for a node
            Node objectNode = getDefaultObjectConfig();
            newRoleContext.addEntry(objectNode.getTypeSystem() + "." + objectNode.getTypeName(), objectNode);
            // Adding the entry
            newRoleContext.addEntry(id, nodeToMerge);
            role_context_.addEntry(role.getId(),newRoleContext);
        }
    }

    /**
     * Loads a given node for the given role into the property context.
     * @param node The node to load.
     * @param role The role for which the node is defined.
     */
    protected void loadNode(Node node, Role role) {
        if(node!=null){
            loadJar(node.getJarList());
            String typeSystem = node.getTypeSystem();
            String typeName = node.getTypeName();
            if(typeSystem != null && !typeSystem.equals("") && typeName != null && !typeName.equals("")){
                mergeConfig(node.getTypeSystem() + "." + node.getTypeName(),node, role);
            }
        }
    }

    /**
     * Loads a given root context for the given role into the property context.
     * @param root The root to load.
     * @param role The role for which the node is defined.
     */
    protected void loadRoot(Root root, Role role) {
        if(root!=null){
            rootMap_.put(role.getId(),root);
        }
    }
    
    /** 
     * Parses the XML file and builds the appropriate context
     */
    protected void buildContext(Browser browser) {
        if (browser != null) {
            // Loads zip elements
            loadZips(browser.getZipList());
            // Loads jar elements
            loadJar(browser.getJarList());
            // Loads root elements 
            loadRoot(browser.getRoot(), default_role_);
            // Loads type-system elements
            List typeSystemList = browser.getTypeSystemList();
            for (Iterator i = typeSystemList.iterator(); i.hasNext();) {
                TypeSystem typeSystem = (TypeSystem) i.next();
                addTypeSystem(typeSystem);
            }
            typeSystemHandler_.addTypeSystem(new JavaTypeSystem());
            // Loads include elements
            List includeList = browser.getIncludeList();
            for (Iterator i = includeList.iterator(); i.hasNext();) {
                Include include = (Include) i.next();
                addInclude(include);
            }
            // Loads node elements
            List nodeList = browser.getNodeList();
            for (Iterator i = nodeList.iterator(); i.hasNext();) {
                loadNode((Node) i.next(), default_role_);
            }
            // Loads role elements
            List roleList = browser.getRoleList();
            for (Iterator i = roleList.iterator(); i.hasNext();) {
                org.objectweb.util.browser.browserConfig.Role role =
                    (org.objectweb.util.browser.browserConfig.Role)i.next();
                // Stores the inherited role and the role.
                String roleId = role.getId().trim();
                if(role.getXmlextends()!=null && !role.getXmlextends().equals(""))
                    roleConfig_.addEntry(roleId,role.getXmlextends());
                Role theRole = new DefaultRole(roleId);
                if(role.getKind().equals("abstract"))
                    theRole.setConcrete(false);
                if(roleMap_.containsKey(roleId)){
                    if(role.getKind().equals("abstract"))
                        ((Role)roleMap_.get(roleId)).setConcrete(false);
                }else{
                    roleMap_.put(roleId,theRole);
                }
                // Loads root elements 
                loadRoot(role.getRoot(), theRole);
                // Loads the nodes defined for this role
                List roleNodeList = role.getNodeList();
                for(Iterator j = roleNodeList.iterator(); j.hasNext();) {
                    loadNode((Node)j.next(), theRole);
                }
            }
        }
    }

    /**
     * Builds the Browser object and builds the context if the created Browser is not null.
     * @param stream A stream opened on the file containing the data defining the Browser
     */
    protected void buildBrowser(InputStream stream) {
        try {
            // We can't validate with DTD at this time (no way to fix the DTD file)
            BrowserBean browser = BrowserBeanImpl.unmarshalBean(new InputStreamReader(stream),false);
            //Browser browser = BrowserUnmarshaller.unmarshal(stream);
            if (browser != null)
                buildContext(browser);
        } catch (java.io.IOException e) {
            System.err.println(e.getMessage());
        }
    }

    /**
     * Builds the Browser object and builds the context if the created Browser is not null.
     * @param file The file containing the data defining the Browser
     */
    protected void buildBrowser(File file) {
        try {
            buildBrowser(new FileInputStream(file));
        } catch (java.io.FileNotFoundException e) {
            System.err.println("Error : " + file + " : File not found !");
        }
    }

    protected PanelFactory getPanelFactory(Panel panel){
        if(panel.getCode()!=null){
        Class panelClass = getClass(panel.getCode());
        if(panelClass!=null) {
            if (org.objectweb.util.browser.api.Panel.class.isAssignableFrom(panelClass)) {
                PanelFactory panelFactory = new DefaultPanelFactory();
                panelFactory.setClassName(panelClass);
                panelFactory.setInheritTypePanel(panel.getInheritTypePanel().equals("false")?false:true);
                return panelFactory;
            } else
                System.err.println(panel.getCode().getValue() + " is not a Panel instance !");
            }
        } else {
            List childList = ((PanelBean)panel).getChildrenList();
            DefaultCompositePanelFactory compositePanelFactory = new DefaultCompositePanelFactory();
            compositePanelFactory.setInheritTypePanel(panel.getInheritTypePanel().equals("false")?false:true);
            Iterator iterator = childList.iterator(); 
            while (iterator.hasNext()) {
                Object element = iterator.next();
                if(Panel.class.isAssignableFrom(element.getClass())){
                    Panel currentPanel = (Panel)element;
                    compositePanelFactory.add(getPanelFactory(currentPanel));
                } else if (Table.class.isAssignableFrom(element.getClass())){
                    Table currentTable = (Table)element;
                    Class actionClass = getClass(currentTable.getCode());
                    if (actionClass!=null){
                        if (org.objectweb.util.browser.api.Table.class.isAssignableFrom(actionClass)) {
                            TableFactory tableFactory = new DefaultTableFactory(tree_);
                            tableFactory.setClassName(actionClass);
                            compositePanelFactory.add((ViewFactory)tableFactory);
                        } else
                            System.err.println(currentTable.getCode().getValue() + " is not an Table instance !");
                    }
                }
            }
            return compositePanelFactory;
        }
        return null;
    }

    /**
     * Added the inherited role of the given roles in a list.
     * @param roles An arrayt of roles.
     * @param roleList The vector in which the inherited roles have to be added.
     */
    protected void buildRole(Role[] roles, Vector roleList){
        // Adds the roles into the list
        for(int i=0;i<roles.length;i++){
            if(!roleList.contains(roles[i])){
                roleList.add(roles[i]);
            }
        }
        // Creates a list containing all the role extended by the given roles array.
        Vector allExtendedRoles = new Vector();
        for(int i=0;i<roles.length;i++){
            String theRoles = (String)roleConfig_.getLocalValue(roles[i].getId());
            if(theRoles!=null){
                StringTokenizer st = new StringTokenizer(theRoles,",");
                while (st.hasMoreTokens()) {
                    Role theRole = (Role)roleMap_.get(st.nextToken().trim());
                    if(theRole!=null && !roleList.contains(theRole))
                        allExtendedRoles.add(theRole);
                }
            }
        }
        // Recursive call
        if(allExtendedRoles.size()!=0)
            buildRole((Role[])allExtendedRoles.toArray(new Role[0]),roleList);
    }


    /**
     * Provides the inherited roles of the given role. If no inherited node is defined, an empty array is return.
     * @param role The role. 
     * @return An array containing the inherited roles for the given role.
     */
    protected Role[] getInheritedRoles(Role[] roles){
        Role[] returnRoleList = null;
        if(sameRoleList_)
            //object = inheritedRolesMap_.getLocalValue(role.getId());
            returnRoleList = current_roles_inheritance_;
        if(returnRoleList==null){
            Vector roleList = new Vector();
            buildRole(roles, roleList);
            roleList.add(default_role_);
            returnRoleList = (Role[])roleList.toArray(new Role[0]); 
            current_roles_inheritance_ = returnRoleList; 
            sameRoleList_ = true;
            //inheritedRolesMap_.addEntry(role.getId(),object);
        }
        return returnRoleList;
    }
    
    /**
     * Provides the IconProvider associated to the given icon.
     * @param icon The icon. It can be an icon file or a class able to provide an icon.
     * @return The associated IconProvider
     */
    protected IconProvider getIconProvider(Icon icon){
        if (icon != null) {
            if(icon.getIconFile()!=null) {
                FileIconProvider iconFactory = new DefaultFileIconProvider();
                iconFactory.setUrl(icon.getIconFile().getUrl());
                return iconFactory;
            } else if (icon.getCode()!=null) {
                try {
                    Class factory = getClass(icon.getCode());
                    if (factory!=null) {
                        if (IconProvider.class.isAssignableFrom(factory))
                            return (IconProvider)factory.newInstance();
                        else
                            System.err.println(icon.getCode().getValue() + " is not an IconProvider instance !");
                    }
                } catch (InstantiationException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
//            } else {
//                System.err.println("An element \"icon-file\" or \"code\" is mandatory in an icon element definition !");
            }
        }
        return null;        
    }
    
    /**
     * Provides the KeyStroke associates to the given Mnemonic. 
     * @param mnemonic The object containing the definition of the keyStroke
     * @return The associated KeyStroke.
     */
    protected KeyStroke getKeyStroke(Accelerator accelerator){
        if(accelerator!=null){
            String car = accelerator.getXmlchar();
            if(car!=null && car.length()>0){
                Character character = new Character(car.charAt(0));
                int modifier = 0;
                modifier = (accelerator.getAlt().equals("true")?java.awt.Event.ALT_MASK:0)
                    |(accelerator.getCtrl().equals("true")?java.awt.Event.CTRL_MASK:0)
                    |(accelerator.getShift().equals("true")?java.awt.Event.SHIFT_MASK:0)
                    |(accelerator.getMeta().equals("true")?java.awt.Event.META_MASK:0);
                if(modifier==0)
                    return KeyStroke.getKeyStroke(character.charValue());
                else
                    return KeyStroke.getKeyStroke(Character.toUpperCase(character.charValue()), modifier);
            }    
        }
        return null;
    }

    /**
     * Provides the character for the given mnemonic.
     * @param mnemonic The given mnemonic.
     * @return The associated character.
     */
    protected Character getMnemonic(Mnemonic mnemonic) {
        if(mnemonic!=null){
            String car = mnemonic.getXmlchar();
            if(car!=null && car.length()>0)
                return new Character(car.charAt(0));
        }
        return null;
    }
    
    // ==================================================================
    //
    // Public methods of XMLDecoder interface.
    //
    // ==================================================================

    /**
     * Fixes the file which contains the properties.
     * @param path The path containing the XML file(s) containing the configuration 
     * we want to abtain under a Context view (It can be a single file or a directory)
     */
    public void setPropertyFile(String path) {
        InputStream inputStream = null;
        if (path != null) {
            // Tries to load the resource from the ClassLoader
            inputStream = ClassResolver.getResourceAsStream(path);
            if(inputStream==null){            
                // Tries to load the resource from the file system
                try {
                    File file = new File(path);
                    if (file.isDirectory()) {
                        File[] files = file.listFiles(new XMLFileFilter());
                        for (int i = 0; i < files.length; i++) {
                            buildBrowser(files[i]);
                        }
                    } else {
                        buildBrowser(file);
                    }           
                } catch(Exception e1) {
                    System.err.println("[BrowserXMLParser.setPropertyFile] " + path + ": Resource not found on the file system ! ");
                }
            }
            if(inputStream!=null) {
                buildBrowser(inputStream);   
            }
        } else
            System.err.println("The path param is null ! ");
    }

    /**
     * Fixes an array of files (or directories) which contains the properties
     * @param files The path containing the XML file(s) containing the configuration 
     * we want to abtain under a Context view (Each value of the array can be a single file or a directory)
     */
    public void setPropertyFile(String[] files) {
        for (int i = 0; i < files.length; i++)
            setPropertyFile(files[i]);
    }

    // ==================================================================
    //
    // Public methods of BrowserProperty interface.
    //
    // ==================================================================

    /**
     * Fixes the context property to use to constructs the wrappers
     * @param properties The context property to use
     */
    public void setContextProperty(ContextProperty properties) {
        contextProperty_ = properties;
        if (contextProperty_ != null)
            decoder_ = contextProperty_.getDecoder();
    }

    /**
     * Provides the TypeSystem to use in order to find appropriate key.
     * @return The TypeSystem to use.
     */
    public org.objectweb.util.browser.core.api.TypeSystem getTypeSystem(){
        return (org.objectweb.util.browser.core.api.TypeSystem)typeSystemHandler_;
    }

    /**
     * Provides the entry to add into the icon property context or null.
     * @param id The research key
     * @return The entry to add or null
     */
    public Entry getIcon(String id, Role role, ExtendedBoolean nodeFound) {
        Node node = getNode(id,role,false);
        if (node != null) {
            Icon icon = node.getIcon();
            if (icon != null) {
                nodeFound.setValue(true);
                IconProvider iconProvider = getIconProvider(icon); 
                if(iconProvider != null)
                    return createEntry(id, iconProvider);
            }
        }
        return null;
    }

    /**
     * Provides the entry to add into the panel property context or null.
     * @param id The research key
     * @return The entry to add or null
     */
    public Entry getPanel(String id, Role role, ExtendedBoolean nodeFound) {
        Node node = getNode(id,role,false);
        if (node != null) {
            Panel panel = node.getPanel();
            if (panel != null) {
                nodeFound.setValue(true);
                PanelFactory panelFactory = getPanelFactory(panel);
                if (panelFactory != null)
                    return createEntry(id, panelFactory); 
            }
        }
        return null;
    }

    /**
     * Provides the entry to add into the menu property context or null.
     * @param id The research key (a class name)
     * @param role The associated role
     * @param nodeFound Indicates if a node has been found or not.
     * @return The entry to add or null
     */
    public Entry getMenu(String id, Role role, ExtendedBoolean nodeFound) {
        Node node = getNode(id,role,false);
        if (node != null) {
            Menu menu = node.getMenu();
            if (menu != null) {
                nodeFound.setValue(true);
                ContextContainer items = new DefaultContextContainer();
                ContextContainer childItems = new DefaultContextContainer();
                List itemList = menu.getItemList();
                int cpt = 0;
                for (Iterator i = itemList.iterator(); i.hasNext();) {
                    Item item = (Item) i.next();
                    ItemProperty ip = new ItemProperty(item.getLabel(),item.getCode().getValue(),item.getTreeChildVisible().equals("false") ? false : true,item.getTypeChildVisible().equals("true") ? true : false, getIconProvider(item.getIcon()),getKeyStroke(item.getAccelerator()),getMnemonic(item.getMnemonic()));
                    String label = item.getLabel();
                    if (label!=null && label.equals("-separator-")){
                        label = label + cpt;
                        cpt++;
                    }
                    if (item.getTreeChildVisible().equals("false"))
                        items.addEntry(label, ip);
                    else
                        childItems.addEntry(label, ip);
                }
                MenuFactoryConfiguration menuFactory = new DefaultMenuFactory();
                menuFactory.setInheritTreeMenu(menu.getInheritTreeMenu().equals("false") ? false : true);
                menuFactory.setInheritTypeMenu(menu.getInheritTypeMenu().equals("false") ? false : true);
                menuFactory.setItems(items);
                menuFactory.setChildItems(childItems);
                return createEntry(id, menuFactory);
            }
        }
        return null;
    }

    /**
     * Provides the entry to add into the wrapper property context or null.
     * @param id The research key
     * @return The entry to add or null
     */
    public Entry getWrapper(String id, Role role, ExtendedBoolean nodeFound) {
        Node node = getNode(id,role,false);
        if (node != null) {
            Wrapper wrapper = node.getWrapper();
            if (wrapper != null) {
                nodeFound.setValue(true);
                Class wrapperClass = getClass(wrapper.getCode());
                if(wrapperClass != null) {
                    if (org.objectweb.util.browser.api.Wrapper.class.isAssignableFrom(wrapperClass) && Context.class.isAssignableFrom(wrapperClass)) {
                        SpecificContextFactory cf = new DefaultContextFactory(decoder_);
                        cf.setClassName(wrapperClass);
                        return createEntry(id, cf);
                    } else
                        System.err.println(wrapper.getCode().getValue() + " is not a Wrapper and a Context instance !");
                }
            }
        }
        return null;
    }

    protected class XMLFileFilter implements java.io.FileFilter {
        public boolean accept(File pathname) {
            String extension = pathname.getName().substring(pathname.getName().lastIndexOf('.') + 1);
            return extension.equalsIgnoreCase("xml") && !pathname.isDirectory();

        }
    }

    /**
     * Provides the entry to add into the dropaction property context or null.
     * @param id The research key
     * @return The entry to add or null
     */
    public Entry getDropAction(String id, Role role, ExtendedBoolean nodeFound) {
        Node node = getNode(id,role,false);
        if (node != null) {
            DropAction dropAction = node.getDropAction();
            if (dropAction != null) {
                nodeFound.setValue(true);
                Class actionClass = getClass(dropAction.getCode());
                if(actionClass != null) {
                    if (org.objectweb.util.browser.api.DropAction.class.isAssignableFrom(actionClass)) {
                        DropActionFactory dropActionFactory = new DefaultDropActionFactory();
                        dropActionFactory.setLabel(dropAction.getLabel());
                        dropActionFactory.setClassName(actionClass);
                        return createEntry(id, dropActionFactory);
                    } else
                        System.err.println(dropAction.getCode().getValue() + " is not a DropAction instance !");
                }
            }
        }
        return null;        
    }

    /**
     * Provides the entry to add into the info property context or null.
     * @param id The research key
     * @return The associated entry or null
     */
    public Entry getInfo(String id, Role role, ExtendedBoolean nodeFound){
        Node node = getNode(id,role,false);
        if (node != null) {
            Info info = node.getInfo();
            if (info != null) {
                nodeFound.setValue(true);
                Class actionClass = getClass(info.getCode());
                if(actionClass !=null) {
                    if (org.objectweb.util.browser.api.Info.class.isAssignableFrom(actionClass)) {
                        InfoFactory infoFactory = new DefaultInfoFactory();
                        infoFactory.setClassName(actionClass);
                        return createEntry(id, infoFactory);
                    } else
                        System.err.println(info.getCode().getValue() + " is not an Info instance !");
                }
            }
        }
        return null;                
    }

    /**
     * Provides the initial context factory for the given role.
     */
    public RootFactory getRoot(Role role){
        if(rootMap_.containsKey(role.getId())){
            Root root = (Root)rootMap_.get(role.getId());
            Class actionClass = getClass(root.getCode());
            if(actionClass !=null) {
                if (org.objectweb.util.browser.api.Context.class.isAssignableFrom(actionClass)) {
                    RootFactory rootFactory = new DefaultRootFactory();
                    rootFactory.setClassName(actionClass);
                    return rootFactory;
                } else
                    System.err.println(root.getCode().getValue() + " is not a Context instance !");
            }
        }
        return null;
    }

    /**
     * Provides the inherited roles of the current role. If no inherited node is defined, an empty array is return.
     * @param role The role. 
     * @return An array containing the inherited roles for the current role.
     */
    public Role[] getInheritedRoles(){
        return getInheritedRoles(current_roles_);
    }
    
    // ==================================================================
    //
    // Public methods of RoleManagement interface.
    //
    // ==================================================================
    
    /** 
     * Provides the current role.
     * 
     * @return The current role. 
     */
    public Role[] getCurrentRoles(){
        return current_roles_;
    }
    
    /** 
     * Fixes the current role.
     * 
     * @param role_id The id of the role.
     */
    public void setCurrentRole(String[] roles_id){
        if(roles_id!=null){
            Role[] roles = new Role[roles_id.length];
            for(int i=0;i<roles_id.length;i++)
                roles[i] = new DefaultRole(roles_id[i].trim());
            setCurrentRole(roles);
        }
        //current_roles_ = new DefaultRole(role_id);
    }

    /**
     * Fixes the current role.
     * 
     * @param role The role to define as current role.
     */
    public void setCurrentRole(Role[] roles){
        sameRoleList_ = false;
        current_roles_ = roles;
    }

    /**
     * Provides the list of all define roles.
     * 
     * @return The list of roles.
     */
    public Role[] getRoleList(){
        return (Role[])roleMap_.values().toArray(new Role[0]);
    }

    // ==================================================================
    //
    // Public method surcharging java.lang.Object methods.
    //
    // ==================================================================
    
    /**
     * Removes the temporary folder containing the extracting files.
     * Overriding methods
     * @see java.lang.Object#finalize()
     */
    public void finalize() throws Throwable {
        if(extractedFolder_.exists()){
            remove(extractedFolder_);
        }
    }
    
 }