/*====================================================================

Objectweb Browser framework
Copyright (C) 2000-2003 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.gui.api;

import org.objectweb.util.browser.api.TreeView;
import org.objectweb.util.browser.core.common.DynamicTree;

/**
 * Used to create new DynamicTree.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * 
 * @version 0.1
 */
public interface TreeProvider {

    //==================================================================
    //
    // No internal state.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods.
    //
    //==================================================================

    /**
     * Creates a new DynamicTree with a same configuration of one contains in the given treeView.
     * Equivalent to a <code>createDynamicTree(treeView,true);</code> call.
     * @param treeView The treeView containinf the configuration of the initial tree.
     * @return The new DynamicTree. 
     * @see org.objectweb.util.browser.core.common.DynamicTree
     */
    public DynamicTree createDynamicTree(TreeView treeView);

    /**
     * Creates a new DynamicTree with a same configuration of one contains in the given treeView.
     * @param treeView The treeView containinf the configuration of the initial tree. 
     * @param withData A boolean to specify if the data of the source tree have to be added into the new one. The default value is "true"
     * @return The new DynamicTree.
     */
    public DynamicTree createDynamicTree(TreeView treeView, boolean withData);

}