/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Offroy.
Contributor(s): ______________________________________.

====================================================================*/
/*
 * Created on 10 juin 2003 by jerome OFFROY (offroy@lifl.fr)
 *
 */

package org.omg.lifl.eclipse.plugin.project.utils.SWT;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.omg.lifl.eclipse.plugin.project.OpenCCM.utils.ProjectMessages;

/**
 * This abstract class is used to make a basic SWT area 
 * which allow configuration of some preferences store in a SetterGroupPreference   
 *
 * @author offroy@lifl.fr
 *
 * @version 0.1
 */
public abstract class DirectoryChooser {

    private Group _parent;
    protected Group subGroup;
    protected boolean subGroupVisible;

    private DirectoryDialog directoryDialog;
    protected Text _text;
    protected Label _label;
    protected Button _btnChooser;
    protected SetterGroupPreference _preference;

    private static final String BLANK_END =
        ProjectMessages.getString("DirectoryChooser.Blank_space");

    /**
     * Constructor
     * @param parent is the SWT parent group
     * @param preference the preferences to manage 
     */
    public DirectoryChooser(Group parent, SetterGroupPreference preference) {
        subGroupVisible = true;
        set_preference(preference);
        directoryDialog = new DirectoryDialog(parent.getShell());
        set_parent(parent);
        groupToSet(get_parent());
    }

    /**
     * create a subGroup which parent is parent_group
     * @param parent_group the main parent group
     */
    public void groupToSet(Group parent_group) {
        String buttonTittle =
            ProjectMessages.getString("DirectoryChooser.BrowseButtonLabel");
        subGroup = new Group(parent_group, SWT.NONE);
        subGroup.setLayout(new RowLayout());
        subGroup.setText(_preference.get_GroupLabel());

        setCheckButton();

        _label = new Label(subGroup, SWT.LEFT);
        _label.setText(_preference.get_Label());
        set_text(new Text(subGroup, SWT.SINGLE | SWT.BORDER));
        _text.setText(_preference.get_Name());
        _text.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                //TODO add verification generic function
                _preference.set_Name(_text.getText().trim());
            }
        });

        directoryDialog.setFilterPath(_preference.get_Name());
        _btnChooser = new Button(subGroup, SWT.NONE);
        _btnChooser.setText(buttonTittle);

        _btnChooser.addListener(SWT.Selection, new Listener() {

            public void handleEvent(Event e) {
                String buffer = directoryDialog.open();
                if (buffer != null)
                    config(buffer);
            }
        });

    }

    /**
     * @param buffer
     */
    protected abstract void config(String buffer);

    /**
     * 
     */
    protected abstract void setCheckButton();

    /**
     * @return the SWT Text widget
     */
    public Text get_text() {
        return _text;
    }

    /**
     * @param text is the SWT Text widget to set
     */
    public void set_text(Text text) {
        _text = text;
    }

    /**
     * @return the SetterGroupPreference which stores preference  
     */
    public SetterGroupPreference get_preference() {
        return _preference;
    }

    /**
     * @param preference is the new SetterGroupPreference 
     */
    public void set_preference(SetterGroupPreference preference) {
        _preference = preference;
    }

    /**
     * @return the parent SWT Group
     */
    public Group get_parent() {
        return _parent;
    }

    /**
     * set the parent SWT Group
     * @param group to set the parent SWT Group
     */
    public void set_parent(Group group) {
        _parent = group;
    }

    /**
     * @return the internal SWT subGroup
     */
    public Group getSubGroup() {
        return subGroup;
    }

    /**
     * set the internal SWT subGroup
     * @param group to set internal SWT subGroup
     */
    public void setSubGroup(Group group) {
        subGroup = group;
    }

    /**
     * @return the visibility of the SWT internal subGroup
     */
    public boolean isSubGroupVisible() {
        return subGroupVisible;
    }

    /**
     * @param b boolean to set visibility of the SWT internal subGroup
     */
    public void setSubGroupVisible(boolean b) {
        subGroupVisible = b;
    }

}
