package ist.coach.coachEmfServicesComponents;

/**
 * @author Administrator (digi@intracom.gr)
 *
 * Copyright (c) Intracom S.A., N.T., 2003
 */

import java.util.Properties;
import java.io.FileInputStream;
import java.util.Enumeration;
import java.util.StringTokenizer;
import org.opennms.protocols.snmp.SnmpObjectId;

public class Configuration {
	
	private static Configuration instance;
	private static final String configFile = "SnmpTrapDaemon.conf";
	private static final String availableTrapsID = "availableTraps";
	private static final String rootObjectsID = "rootObjects";
	public SnmpTrapConfigData[] availableTraps;
	public String[] rootObjects;
	
	
	public static Configuration getInstance() {
		if (instance == null) {
			instance = new Configuration();
			if (instance.configSanity() == false)
				return null;
		}
			
		return instance;
	}
	
	private Configuration() {
		
		Properties props = new Properties();
		try {
			System.out.println("Loading file " + configFile);
			props.load(Configuration.class.getResourceAsStream(configFile));
		}
		catch (Exception e) {
			e.printStackTrace();
			return;
		}
		
		String availableTrapsString = props.getProperty(availableTrapsID);
		String rootObjectsString = props.getProperty(rootObjectsID);
		
		StringTokenizer st = new StringTokenizer(availableTrapsString, " ");
		int numOfTokens = st.countTokens();
		
		this.availableTraps = new SnmpTrapConfigData[numOfTokens];
		
		int counter = 0;
		
		while (st.hasMoreTokens()) {
			
			this.availableTraps[counter] = new SnmpTrapConfigData();
			this.availableTraps[counter].id = new String(st.nextToken().trim());
			
			counter ++;
			
		}
		
		st = new StringTokenizer(rootObjectsString, " ");
		numOfTokens = st.countTokens();
		
		this.rootObjects = new String[numOfTokens];
		
		counter = 0;
		
		while (st.hasMoreTokens()) {
			this.rootObjects[counter] = new String(st.nextToken().trim());
		}
		
		Enumeration en = props.propertyNames();
		
		while (en.hasMoreElements()) {
			
			String name = (String) en.nextElement();
			String value = props.getProperty(name);
			
			for (int i = 0; i < this.availableTraps.length; i++) {
				
				if (name.startsWith(this.availableTraps[i].id)) {
					
					if (name.endsWith("_objectclass")) {
						this.availableTraps[i].objectclass = new String(value);
					} else
					if (name.endsWith("_oid")) {
						this.availableTraps[i].oid = new SnmpObjectId(value);
			
					} else
					if (name.endsWith("_identifier")) {
						this.availableTraps[i].identifier = Integer.parseInt(value);
					}
					else
					if (name.endsWith("_valueIdentifier"))
						this.availableTraps[i].valueOid = new SnmpObjectId(value);
				}	
			}
		}
	}

	private boolean configSanity() {
		
		for (int i = 0; i < this.availableTraps.length; i++) {
			
			if ( (this.availableTraps[i].objectclass == null) ||
				(this.availableTraps[i].oid == null) || 
				(this.availableTraps[i].identifier == -1)) 
				return false;
		}
		
		return true;
		
	}
}
