/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Romain Rouvoy, Areski Flissi.

====================================================================*/

package org.objectweb.ccm.CORBA;

/**
 * This class encapsulates the OpenORB CORBA::ORB singleton object.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @version 0.3
 */ 
public abstract class TheORB
       extends TheORBSingleton
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    private
    TheORB()
    {
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    /**
     * Remove ORB arguments.
     *
     * @param args The command-line arguments.
     *  
     * @return The command line arguments without OpenORB options.
     */
/*
 * Commented for Task #1607 (support for OpenORB 1.4.0 beta1)
 * Philippe Merle
 
    protected static String[]
    remove_ORB_arguments(String[] args)
    {
        int nb = 0;
        
        // Copy arguments without ORB ones.
        String[] tmp = new String[args.length];
        for(int i=0; i<args.length; i++)
        {
            if(args[i].startsWith("-ORBInitRef"))
            {
                i++;
                
                String arg = args[i];
                
                int index = arg.indexOf('=');
                String name = arg.substring(0, index);
                String ior = arg.substring(index+1);
                ((org.openorb.orb.core.ORB)getORB())
                    .addInitialReference(name, string_to_object(ior));
            }
            else if(!args[i].startsWith("-ORB"))
            {
                tmp[nb] = args[i];
                nb++;
            }
        }
        
        // Construct the returned array.
        args = new String[nb];
        for(int i=0; i<nb; i++)
        {
            args[i] = tmp[i];
        }
        
        // Return arguments without ORB ones.
        return args;
    }
*/
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * Initialize the OpenORB CORBA::ORB singleton object.
     *
     * @param args The command line arguments.
     *
     * @return The command line arguments without OpenORB options.
     */
    public static String[]
    initialize(String[] args)
    {
        // Define system properties.
        java.util.Properties properties = System.getProperties();
/*
 * Commented for Task #1607 (support for OpenORB 1.4.0 beta1)
 * Philippe Merle
 
        properties.put("org.omg.CORBA.ORBClass", "org.openorb.orb.core.ORB");
        properties.put("org.omg.CORBA.ORBSingletonClass", "org.openorb.orb.core.ORBSingleton");
*/

        // Initialize the CORBA::ORB.
        setORB( org.omg.CORBA.ORB.init(args, properties) );

        // Manage CORBA initial references provided as Multicast
        // Address group (eg 224.0.0.100)
        
        for (int i = 0; i < args.length; i++) {
            if (args[i].equals("-ORBInitRef")) {
                i++;
                String arg = args[i];
                int index = arg.indexOf('=');
                String location = arg.substring(index + 1);

                if (location.startsWith("mcast:")) {

                // Store the IOR obtained from the MULTICAST server
                // in the command line arguments.
                args[i] =
                arg.substring(0, index) + '=' + TheORBSingleton.request_ior(location);
                }

            }
        }
        
        return remove_ORB_arguments(args) ;
    }
    
    /**
     * Destroy the CORBA::ORB singleton object.
     */
    public static void
    destroy()
    {
        // Not provided by OpenORB!!!
        // getORB().destroy();
    }
}
