package org.coach.tracing.server.viewer;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;

public class MessageGui extends JComponent  {
    public final static Integer NW_SE = new Integer(0);
    public final static Integer NE_SW = new Integer(1);
    public final static Integer SE_NW = new Integer(2);
    public final static Integer SW_NE = new Integer(3);
    public final static Integer UP = new Integer(4);
    public final static Integer DOWN = new Integer(5);
    public final static int arrow_size = 6;
    public final static int delta = 2;
    //the thickness of the message line when calculating the polygon
    public final static Color normal_color = Color.blue;
    public final static Color trail_color = Color.magenta;
    public final static Color text_color = Color.black;
    private static Diagram view;
    private static ScaleManager sm;

    private EventGui start;
    private EventGui end;
    private Integer orientation = null;
    private Point[] arrow_head = new Point[3];
    //array of 3 points
    private Polygon poly = null;
    private Dimension text_dimension = null;
    private Point text_origin = null;
    private String text = "";
    private int identityGuiWidth;

    public MessageGui(Diagram v, EventGui e1, EventGui e2, int identityGuiWidth) {
        super();
        if (e1.getPosition() < e2.getPosition()) {
            start = e1;
            end = e2;
        } else {
            start = e2;
            end = e1;
        }
        this.identityGuiWidth = identityGuiWidth;
        if (view == null) {
            view = v;
            sm = view.getScaleManager();
        }
        text = start.getOpName();
        setLocation(calcOrigin(start.getCenter(), end.getCenter(), start.getRadius()));
        setSize(calcDimension(start.getCenter(), end.getCenter(), start.getRadius()));
        orientation = calcOrientation(start.getCenter(), end.getCenter());
        poly = calcPolygon(start.getCenter(), end.getCenter());
        calcArrow(arrow_head);
        calcText();
        ToolTipManager.sharedInstance().registerComponent(this);
        setToolTipText(text);
    }

    public void paint(Graphics g) {
        //paint the event
        super.paint(g);
        float scale = sm.getScale();
        Graphics2D g2 = (Graphics2D)g;
        g2.setRenderingHints(sm.getHints());

        //set dashed stroke if the message is a return one
        if (start.isReturn()) {
            g2.setStroke(sm.getStrokeDashed());
        } else {
            g2.setStroke(sm.getStroke());
        //draw the lines
        }
        int x1 = 0;
        int y1 = 0;
        int x2 = 0;
        int y2 = 0;

        if ((orientation == NW_SE) || (orientation == SE_NW)) {
            x2 = getWidth() - 1;
            y2 = getH() - 1;
        }

        if ((orientation == NE_SW) || (orientation == SW_NE)) {
            y1 = getH() - 1;
            x2 = getWidth() - 1;
        }

        // color the message if the start and end event is also colored
        if (start.getColor() != null && end.getColor() != null) {
            g2.setColor(start.getColor());
        } else {
            g2.setColor(normal_color);
        }

        if ((orientation == UP) || (orientation == DOWN)) {
            //draw the line to self
            g2.draw(new Line2D.Float(0, 0, (getWidth() - 1) / 2, 0));
            g2.draw(new Line2D.Float((getWidth() - 1) / 2, 0, (getWidth() - 1) / 2, getH() - 1));
            g2.draw(new Line2D.Float(0, getH() - 1, (getWidth() - 1) / 2, getH() - 1));
        } else {
            //draw the line
            g2.draw(new Line2D.Float(x1, y1, x2, y2));
        }
        g2.setStroke(sm.getStroke());

        //draw the arrow head
        if (arrow_head[0] != null) {
            g2.draw(new Line2D.Float(arrow_head[0].x, arrow_head[0].y, arrow_head[1].x, arrow_head[1].y));
            g2.draw(new Line2D.Float(arrow_head[0].x, arrow_head[0].y, arrow_head[2].x, arrow_head[2].y));
        }
        //draw the message text
        g2.setFont(sm.getFont());
        g2.setColor(Color.white);
        String t = text;
        t = getTrimmedText(text, g2);
        g2.setColor(text_color);       
        g2.drawString(t, text_origin.x, text_origin.y);
    }

    private void calcText() {
        double W = getWidth();
        double H = getH();
        text_dimension = sm.getStringDimension(text + "_", this);
        double x = (W - text_dimension.width) / 2;

        if (orientation == DOWN || orientation == UP) {
            x = 0;
        }
        double y = H / 2 + text_dimension.height / 3;

        //the y is at the base line of the string
        if (x < 0) {
            x = 0;
        }
        text_origin = new Point((int)x, (int)y);
    }

    private String getTrimmedText(String s, Graphics2D g) {
        Dimension td = sm.getStringDimension(s, g);
        double W = getWidth();

        if (td.width > W && (s.length() > 0)) {
            while (td.width > W && (s.length() > 0)) {
                s = s.substring(0, s.length() - 1);
                //
                td = sm.getStringDimension(s + "____", this);
            //using '_' character, because something goes wrong with the '.' when measuring !!!
            }
            s += "...";
        }
        return s;
    }

    private void calcArrow(Point[] pl) {
        float scale = sm.getScale();
        double px, py;
        //head
        double x1, y1;
        //-a
        double x2, y2;
        //+a
        double W = getWidth();
        double H = getH();
        double Z = Math.sqrt(W * W + H * H);
        double sin_a = H / Z;
        double cos_a = W / Z;
        double sin_30 = 0.5;
        double cos_30 = Math.sqrt(3) / 2;
        //sin(x)cos(y)-cos(x)sin(y)
        double sin_a_minus_30 = sin_a * cos_30 - cos_a * sin_30;
        //cos(x)cos(y)+sin(x)sin(y)
        double cos_a_minus_30 = cos_a * cos_30 + sin_a * sin_30;
        //sin(x)cos(y)+cos(x)sin(y)
        double sin_a_plus_30 = sin_a * cos_30 + cos_a * sin_30;
        //cos(x)cos(y)-sin(x)sin(y)
        double cos_a_plus_30 = cos_a * cos_30 + sin_a * sin_30;
        x1 = (arrow_size + 1) * scale * cos_a_minus_30;
        y1 = (arrow_size + 1) * scale * sin_a_minus_30;
        x2 = arrow_size * scale * cos_a_plus_30;
        y2 = arrow_size * scale * sin_a_plus_30;

        if (orientation == NW_SE) {
            pl[0] = new Point((int)(W - 1), (int)(H - 1));
            pl[1] = new Point((int)(W - x1 - 1), (int)(H - y1 - 1));
            pl[2] = new Point((int)(W - x2 - 1), (int)(H - y2 - 1));
        }

        if (orientation == NE_SW) {
            pl[0] = new Point(0, (int)(H - 1));
            pl[1] = new Point((int)(x1), (int)(H - y1 - 1));
            pl[2] = new Point((int)(x2), (int)(H - y2 - 1));
        }

        if (orientation == DOWN) {
            pl[0] = new Point(0, (int)(H - 1));
            pl[1] = new Point((int)((scale * arrow_size * Math.sqrt(3) / 2)), (int)(H + (scale * arrow_size / 2) - 1));
            pl[2] = new Point((int)((scale * arrow_size * Math.sqrt(3) / 2)), (int)(H - (scale * arrow_size / 2) - 1));
        }

        //rare, probably wrong cases, we do not show arrows at all for them
        if (orientation == UP) {
            pl[0] = null;
            pl[1] = null;
            pl[2] = null;
        }

        if (orientation == SW_NE) {
            pl[0] = null;
            pl[1] = null;
            pl[2] = null;
        }

        if (orientation == SE_NW) {
            pl[0] = null;
            pl[1] = null;
            pl[2] = null;
        }
    }

    private Point calcOrigin(Point p1, Point p2, int r) {
        //calculate the origin
        double W = Math.abs(p2.x - p1.x);

        if (W < 3) {
            return new Point((int)(Math.min(p1.x, p2.x) + r), (int)Math.min(p1.y, p2.y));
        } else {
            double H = Math.abs(p2.y - p1.y);
            double Z = Math.sqrt(W * W + H * H);
            double rx = r * W / Z;
            double ry = r * H / Z;
            return new Point((int)(Math.min(p1.x, p2.x) + rx + 2), (int)(Math.min(p1.y, p2.y) + ry));
        }
    }

    private Dimension calcDimension(Point p1, Point p2, int r) {
        //calculate the size
        float scale = sm.getScale();
        double W = Math.abs(p2.x - p1.x);

        if (W < 3) {
//            int ww = (int)((view.getIdentityGui(start.getEventKey()).getWidth()));

            int ww = identityGuiWidth;
            return new Dimension(ww, (int)(Math.abs(p1.y - p2.y) + (scale * arrow_size / 2 + 1)));
        //      return new Dimension((int)Math.abs(p1.y - p2.y), (int)(Math.abs(p1.y - p2.y) + (scale * arrow_size / 2 + 1)));
        } else {
            double H = Math.abs(p2.y - p1.y);
            double Z = Math.sqrt(W * W + H * H);
            double rx = r * W / Z;
            double ry = r * H / Z;
            //adjustment for the arrow head
            return new Dimension((int)(W - (2 * rx)), (int)((H - 2 * ry) + (scale * arrow_size / 2 + 1)));
        }
    }

    private Integer calcOrientation(Point p1, Point p2) {

        //calculate the arrow orientation
        if (p1.x < p2.x) {

            //W - E
            if (p1.y < p2.y) {
                //N - S
                return NW_SE;
            } else {
                //S - N
                return SW_NE;
            }
        } else {

            if ((p1.x - p2.x) < 3) //checking for self arrows
                                   {

                //we have an arrow to self
                if (p1.y < p2.y) {
                    //UP
                    return DOWN;
                } else {
                    //DOWN
                    return UP;
                }
            }
            {

                //E - W
                if (p1.y < p2.y) {
                    //N - S
                    return NE_SW;
                } else {
                    //S - N
                    return SE_NW;
                }
            }
        }
    }

    private Polygon calcPolygon(Point p1, Point p2) {
        Polygon p = new Polygon();
        //draw the lines
        int x1 = 0;
        int y1 = 0;
        int x2 = 0;
        int y2 = 0;

        if ((orientation == NW_SE) || (orientation == SE_NW)) {
            x2 = getWidth() - 1;
            y2 = getH() - 1;
        }

        if ((orientation == NE_SW) || (orientation == SW_NE)) {
            y1 = getH() - 1;
            x2 = getWidth() - 1;
        }

        if ((orientation == UP) || (orientation == DOWN)) {
            p.addPoint(0, -delta);
            p.addPoint((getWidth() - 1) / 2 + delta, -delta);
            p.addPoint((getWidth() - 1) / 2 + delta, getH() - 1 + delta);
            p.addPoint(0, getH() - 1 + delta);
            p.addPoint(0, getH() - 1 - delta);
            p.addPoint((getWidth() - 1) / 2 - delta, getH() - 1 - delta);
            p.addPoint((getWidth() - 1) / 2 - delta, +delta);
            p.addPoint(0, +delta);
        } else {
            float scale = sm.getScale();
            double W = getWidth() - 1;
            double H = getH() - 1;
            double Z = Math.sqrt(W * W + H * H);
            double sin_a = H / Z;
            double cos_a = W / Z;
            double dx = (delta * scale) * sin_a;
            double dy = (delta * scale) * cos_a;

            if ((orientation == NW_SE) || (orientation == SE_NW)) {
                p.addPoint((int)(x1 - dx), (int)(y1 + dy));
                p.addPoint((int)(x1 + dx), (int)(y1 - dy));
                p.addPoint((int)(x2 + dx), (int)(y2 - dy));
                p.addPoint((int)(x2 - dx), (int)(y2 + dy));
            }

            if ((orientation == NE_SW) || (orientation == SW_NE)) {
                p.addPoint((int)(x1 - dx), (int)(y1 - dy));
                p.addPoint((int)(x1 + dx), (int)(y1 + dy));
                p.addPoint((int)(x2 + dx), (int)(y2 + dy));
                p.addPoint((int)(x2 - dx), (int)(y2 - dy));
            }
        }
        return p;
    }

    public int getH() {
        float scale = sm.getScale();
        return (int)(getHeight() - (scale * arrow_size / 2));
    }
}
