/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.Deployment;

public class ServerActivatorMain  extends org.objectweb.ccm.CORBA.ApplicationServer
{
    
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    /**
     ** To store the component server name.
     **/
    protected String name_ ="ServerActivator";
    
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
    
    /**
     **  The constructor.
     **/
    public
    ServerActivatorMain()
    {
    }
    
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     **  The bootstrap main method.
     **
     **  @param args The command-line arguments.
     **/
    public static void
    main(String args[])
    {
    ServerActivatorMain server = new ServerActivatorMain();
    int status = server.start(args);
    System.exit(status);
    }
    
    // ==================================================================
    //
    // Methods for the org.objectweb.ccm.CORBA.ApplicationServer.
    //
    // ==================================================================
    
    /**
     **  Before running the CORBA::ORB.
     **
     **  @param args The command line arguments.
     **         args[0] nom de la machine locale
     **         args[1]  non du fichier a ecrire
     ** 
     **  @return The status.
     **/
    public int
    before_run(String[] args)
    {
        // System.err.println("args[0]:"+args[0]+"args[1]:"+args[1]+"args[2]:"+args[2]+"args[3]:"+args[3]);
    name_=name_+args[2];    
    // Init the OpenCCM Components runtime.
    org.objectweb.ccm.Components.Runtime.init();
    
    // Create the server servant.
    ServerActivatorImpl afi=null;
    if(args[1].equalsIgnoreCase("pda_true"))
        afi = new ServerActivatorImpl(args[2],args[3],true);
    else afi = new ServerActivatorImpl(args[2],args[3]);
    
    // Activate it.
    org.omg.CORBA.Object ref =
        afi._this_object(org.objectweb.ccm.CORBA.TheORB.getORB());
    
    org.omg.Components.Deployment.ServerActivator serverRef = 
        org.omg.Components.Deployment.ServerActivatorHelper.narrow(ref);
    
    // Obtain the Name Service.
    org.objectweb.ccm.CORBA.NamingContext ns =
        org.objectweb.ccm.CORBA.TheNameService.getNamingContext();

    // Bind the server object in the name service.
    ns.rebind(name_, serverRef);
    
    // Output the IOR of the ServerActivator.
    org.objectweb.ccm.CORBA.TheORB.save_IOR(serverRef, args[0]);
    
    try{
            //setup componentinstallation for this JVM 
        org.omg.CosNaming.NameComponent[] ncomp
                = new org.omg.CosNaming.NameComponent[1];
        
        ncomp[0]= new org.omg.CosNaming.NameComponent("ComponentInstallation"+args[2],"");
        
        org.omg.CORBA.Object obj = org.objectweb.ccm.CORBA.TheNameService
                                            .getNamingContext().getNamingContext().resolve(ncomp);
        
        org.omg.Components.Deployment.ComponentInstallation server_inst=
        org.omg.Components.Deployment.ComponentInstallationHelper.narrow(obj);
        
        
        server_inst.install("openccm_plugins", "file:"+args[3]+"/jar/OpenCCM_Plugins.jar");             
        
        TheComponentInstallation.setComponentInstallation(server_inst);
        }
    catch(Exception e){ e.printStackTrace();}       
    
    // Return OK.
    System.out.println("The OpenCCM Server Activator ["+name_+"] is ready.");
    return 0;
    }
    
    /**
     **  After running the CORBA::ORB.
     **/
    public void
    after_run()
    {
    // Obtain the Name Service.
    org.objectweb.ccm.CORBA.NamingContext ns =
        org.objectweb.ccm.CORBA.TheNameService.getNamingContext();

    // Unbind the server object from the name service.
    ns.unbind(name_);    
    }    
    
}
