/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base;

//Package dependencies.
import java.util.Hashtable;
import org.objectweb.apollon.framework.Bean;

/**
 * The BaseHandlerResolver is a basic implementation to resolve the binding between
 * an appolon bean and the package where is the deployer in charge to manage it.
 *
 * @author <a href="mailto:briclet@lifl.fr">Frederic Briclet</a>
 *
 * @version 0.1
 */
public class BaseHandlerResolver 
    implements HandlerResolver 
{
    //  ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    //Internal table to memorise the binding
    private Hashtable convertion;

    //  ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    /**
     * BaseHandlerResolver which initialize the resolution table.
     */
    public BaseHandlerResolver() {
        convertion = new Hashtable();

        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.ComponentsupportedinterfaceBeanImpl",
            "ccm.deployer.binder.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.ConnecteventBeanImpl",
            "ccm.deployer.binder.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.ConnectinterfaceBeanImpl",
            "ccm.deployer.binder.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.ConnectionsBeanImpl",
            "ccm.deployer.binder.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.ConsumesportBeanImpl",
            "ccm.deployer.binder.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.EmitsportBeanImpl",
            "ccm.deployer.binder.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.ExistinghomeBeanImpl",
            "ccm.deployer.installer.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.ExistinginterfaceBeanImpl",
            "ccm.deployer.binder.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.HomeinterfaceBeanImpl",
            "ccm.deployer.binder.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.ProvidesportBeanImpl",
            "ccm.deployer.binder.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.PublishesportBeanImpl",
            "ccm.deployer.binder.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.TraderqueryBeanImpl",
            "ccm.deployer.binder.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.UsesportBeanImpl",
            "ccm.deployer.binder.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.ComponentfileBeanImpl",
            "ccm.deployer.installer.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.ComponentfilesBeanImpl",
            "ccm.deployer.installer.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.ComponentinstantiationBeanImpl",
            "ccm.deployer.installer.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.ComponentplacementBeanImpl",
            "ccm.deployer.installer.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.ComponentpropertiesBeanImpl",
            "ccm.deployer.installer.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.DestinationBeanImpl",
            "ccm.deployer.installer.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.HomeplacementBeanImpl",
            "ccm.deployer.installer.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.HostcollocationBeanImpl",
            "ccm.deployer.installer.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.PartitioningBeanImpl",
            "ccm.deployer.installer.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.ProcesscollocationBeanImpl",
            "ccm.deployer.installer.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.NodeBeanImpl",
            "ccm.deployer.installer.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.RegistercomponentBeanImpl",
            "ccm.deployer.registrar.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.RegisterwithhomefinderBeanImpl",
            "ccm.deployer.registrar.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.RegisterwithnamingBeanImpl",
            "ccm.deployer.registrar.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.RegisterwithtraderBeanImpl",
            "ccm.deployer.registrar.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.ComponentassemblyBeanImpl",
            "ccm.deployer.root.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.FileinarchiveBeanImpl",
            "ccm.deployer.util.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.FindbyBeanImpl",            
            "ccm.deployer.util.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.componentassembly.beans.LinkBeanImpl",            
            "ccm.deployer.util.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.softpkg.beans.CodeBeanImpl",
            "ccm.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.softpkg.beans.DependencyBeanImpl",
            "ccm.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.softpkg.beans.FileinarchiveBeanImpl",
            "ccm.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.softpkg.beans.ImplementationBeanImpl",
            "ccm.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.softpkg.beans.SoftpkgBeanImpl",
            "ccm.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.softpkg.beans.DescriptorBeanImpl",
            "ccm.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.softpkg.beans.LinkBeanImpl",
            "ccm.Deployer");
        convertion.put(
            "org.objectweb.ccm.descriptor.corbacomponent.beans.CorbacomponentBeanImpl",
            "ccm.Deployer");

    }
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public methods for AN_INTERFACE.
    //
    // ==================================================================
    
    /**
     * The getExtension method provided the relative package path of the 
     * deployer in charge of the @param bean element kind. The root of this
     * relative path is the org.objectweb.descriptor package.
     * Note: This method is usefull to use the appollon handler mecanisme.
     * 
     * @param bean The bean kind which requires the deployer.
     * @return The relative path needed to find the deployer with the extension 
     * manager mechanism
     */
    public String 
    getExtension(Bean bean) 
    {
        return (String) convertion.get(bean.getClass().getName());
    }

}
