/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.binder;

import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.util.CannotResolveReferenceException;
import org.omg.CORBA.Object;
import org.omg.Components.CCMObject;
import org.omg.Components.Cookie;
import org.omg.Components.EventConsumerBase;
import org.omg.Components.ExceededConnectionLimit;
import org.omg.Components.InvalidConnection;
import org.omg.Components.InvalidName;

/**
 * The Publishesport deployer is used to resolve a publishesport and 
 * connect the consumer to it.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class PublishesportDeployer 
       extends PublishesportDeployerContext 
    {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    private CCMObject publisher;
    private Cookie cookie;
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    /**
     * Internal method to resolve the CCMObject which supports the
     * publishesport
     * @return the CCMObject supporting the publishesport
     * @throws CannotResolveException if the CCMObject cannot be found
     */
    private org.omg.Components.CCMObject 
    getCCMObjectPublisher() 
    throws CannotResolveReferenceException
    {
        Object ccm;
        
        if(getComponentinstantiationDeployer()!=null)
            ccm=getComponentinstantiationDeployer().getComponent();
        else ccm=getFindbyDeployer().resolveReference(); 
        
        return org.omg.Components.
              CCMObjectHelper.narrow(ccm);
    }    
    
    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================
    
    /**
     * Connect the given @param consumedObject consumer to the current
     * publishesport
     * @param consumedObject the consumer to connect
     * @throws InvalidName if the connection name is invalid
     * @throws InvalidConnection if the given consumer is invalid
     * @throws ExceededConnectionLimit if too many consumer are already connect
     * @throws CannotResolveReferenceException if the precised CCMobject cannot be founded
     */           
    public Cookie 
    connectPublisherToConsumer(EventConsumerBase consumedObject ) 
    throws InvalidName, 
           InvalidConnection, 
           ExceededConnectionLimit, 
           CannotResolveReferenceException
    {
        
        getLogger().log(">> Attempt to establish an event connection on port :"
                                                            +getPublishesport()
                                                            .getPublishesidentifier()
                                                            .getValue());
        
        publisher=getCCMObjectPublisher();

        cookie=publisher.subscribe(getPublishesport()
                                    .getPublishesidentifier()
                                    .getValue()
                                     ,consumedObject);
        getLifeCycleManager().stepEndedSuccessFully();
        return cookie;                                        
    }
    
    /**
     * DisconnectConsumer is used to disconnect the previous connnected consumer
     * from the current publisher.
     * 
     * @throws InvalidName if the port name is invalid
     * @throws InvalidConnection if current cookie is invalid
     */
    public void 
    disconnectConsumer() 
    throws InvalidName, 
           InvalidConnection
    
    {
        publisher.unsubscribe(getPublishesport()
                            .getPublishesidentifier()
                            .getValue(),cookie);
        getLifeCycleManager().stepEndedSuccessFully();
    }
}
