/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.installer;

// Local Interface Import Statements
import org.objectweb.ccm.descriptor.componentassembly.beans.ComponentfileBeanImpl;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.*;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.util.FileinarchiveDeployer;
import org.objectweb.ccm.descriptor.softpkg.beans.ImplementationBean;
import org.objectweb.ccm.descriptor.softpkg.beans.SoftpkgBeanImpl;
import org.objectweb.ccm.descriptor.softpkg.ccm.ImplementationDeployer;
import org.objectweb.ccm.descriptor.softpkg.ccm.SoftpkgDeployer;
import org.objectweb.ccm.descriptor.softpkg.ccm.SoftpkgDeployerHandler;
import org.objectweb.ccm.descriptor.softpkg.ccm.SoftpkgRootDeployerContext;

/**
 * This handler is used to produce componentfiledeployer from an
 * componentfile bean .
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class ComponentfileDeployerHandler 
       extends HandlerBase 
    {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    private ComponentfileDeployer _deploy;
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================
    /**
     * This initialization method is used to check that all the services and 
     * information required to deploy are presents. If all is OK the deployer 
     * is put in inactive deployer state waiting for the deployment processing.
     * 
     * @exception InitializationError is thrown if a problem is detected 
     */       
    public Object 
    getInstance(ComponentfileBeanImpl comp)
    throws InitializationError 
    {
        if (_deploy == null) {
            _deploy = new ComponentfileDeployer();
            _deploy.connectComponentfile(comp);
            applyCommonConfig(_deploy);

            if (comp.getFileinarchive() != null)
                _deploy.connectFileinarchiveDeployer(
                    (FileinarchiveDeployer) getDeployer(comp
                        .getFileinarchive()));

            _deploy.initialize();

            SoftpkgDeployer softDeployer =
                (SoftpkgDeployer) getDeployer2(_deploy.getSoftpkg());
            _deploy.connectSoftpkgDeployer(softDeployer);

            ImplementationBean[] ib = _deploy.getImplementations();

            for (int i = 0; i < ib.length; i++) {
                ImplementationDeployer id =
                    (ImplementationDeployer) getDeployer(ib[i]);
                _deploy.connectImplementationDeployer(id);
            }
        }
        return (_deploy);
    }
    
    public Object getDeployer2(Object obj)
    throws InitializationError 
    {        
        try{ 
            SoftpkgBeanImpl sbi=(SoftpkgBeanImpl)obj;
            SoftpkgDeployerHandler tmp =
                (SoftpkgDeployerHandler) 
                    sbi.getExtensionManager()
                       .getExtensionByName(resolveExtention(sbi));

            SoftpkgRootDeployerContext srdc=new SoftpkgRootDeployerContext();
            srdc.connectZipEntryRetriever
                (_deploy.getFileinarchiveDeployer().getInnerArchive());
            
            srdc.connectGlobalDeployerContext(getRootDeployerContext());        
            tmp.connectRootDeployerContext(srdc);
            tmp.connectHandlerContext(getHandlerContext());

            return tmp.getInstance(sbi);
        }
        catch(Exception e){
            e.printStackTrace();
            throw new InitializationError(_deploy,e," Failed during softpkg retrieval");
        }
     }
}
