/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.installer;


//Import all the required packages
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.*;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.registrar.*;
import org.omg.Components.CCMObject;

/**
 * ComponentinstantiationDeployer is used to deploy a component
 * instance.
 *
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */
public class ComponentinstantiationDeployer
       extends ComponentinstantiationDeployerContext
    {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    private CCMObject component;
    // ==================================================================
    //
    // Internals methods.
    //
    // ==================================================================
    /**
     * Internal method to setup the componentproperties precised
     * 
     * @param new_component the component to setup
     * @throws FatalDeploymentException if a fatal problem occured during the 
     * properties setting.
     */
    private void 
    setupComponentProperties(org.omg.Components.CCMObject new_component)
    throws FatalDeploymentException
   {
        try{
            getComponentpropertiesDeployer()
            .setupComponentProperties(new_component);
        }
        catch(ComponentPropertiesSetupFailureException e){
             getErrorManager().submitException(e);   
        }
      
    }
    /**
     * Internal method to make all the component registration
     * 
     * @param new_component to register
     * @throws FatalDeploymentException is a problem occured during component registration
     */
    private void 
    registerComponent(org.omg.Components.CCMObject new_component)
    throws FatalDeploymentException 
    {
        traverse(this,getRegisterwithcomponentDeployers());
    }
    
    /**
     * Registration visitor used to invoke registration on children 
     * registration deployers.
     * 
     * @param deployer The registration deployer
     * @param status The deployer status
     * @throws FatalDeploymentException is a fatal error occured during registration
     */
    public void visit(RegistercomponentDeployer deployer, DeployerInactiveState status)
    throws FatalDeploymentException
    {        
         deployer.makeComponentRegistration(component);
    }
    
        
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================


    /**
     * The deploy method goals are multiples. It must instantiate 
     * the component, setup their property with the cpf file linked 
     * and finally register the component in the define services.
     * @param homeDeployed the home to use for instiation component.
     * @throws FatalDeploymentException is a fatal problem occured
     * @throws ComponentInstantiationFailureException if the component 
     * cannot be instantiated.
     */    
    public void 
    deploy(org.omg.Components.CCMHome homeDeployed)
    throws FatalDeploymentException,
           ComponentInstantiationFailureException 
    {
        try{
            //instantiate the component
             component =
                org.omg.Components.KeylessCCMHomeHelper
                .narrow(homeDeployed)
                .create_component();
            }
         catch(Exception e){
                getLifeCycleManager().stepAborded();
                throw new ComponentInstantiationFailureException(this,e);
        }
            
        // Test if a property file is define
        if(getComponentpropertiesDeployer()!=null)                    
                setupComponentProperties(component);
        try{           
            //Register component in registration service
            registerComponent(component);
            storeComponentReference(component);
        }
        catch(Exception e){ e.printStackTrace();}
        
        getLifeCycleManager().stepEndedSuccessFully();
    }
 
    /**
     * Instantiated component accessor.
     * 
     * @return the component instantiated.
     */
    public CCMObject 
    getComponent()
    {
        return component;
    }
        
    /**
     * Remove the component refence and unbind it
     * 
     *@throws FatalDeploymentException if a fatal problem occured
     */
    public void 
    tearDown()
    throws FatalDeploymentException
    {        
        if(getLifeCycleManager().getStatus()
             instanceof DeployerFailureException)
         {
             getLifeCycleManager().stepEndedSuccessFully();
             return ;
         }
        
        traverse(this,getRegisterwithcomponentDeployers());
        try{
            unstoreComponentReference();
            component.remove();
        }
        catch(Exception e){
            //getErrorManager().submitException(e);
        }
        getLifeCycleManager().stepEndedSuccessFully();     
    }
    
    /**
     * The component unregistration visitor.
     *  
     * @throws FatalDeploymentException if a fatal problem occured
     */
    public void 
    visit(RegistercomponentDeployer deployer, DeployerActiveState status)
    throws FatalDeploymentException
    {
        deployer.undoComponentRegistration();    
    }
    
    
   
         
}
