/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.installer;

import java.util.LinkedList;

import org.objectweb.ccm.descriptor.componentassembly.Homeplacement;
import org.objectweb.ccm.descriptor.componentassembly.beans.HomeplacementBean;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.ChildDeployerContext;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.InitializationError;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.registrar.*;
import org.objectweb.ccm.descriptor.softpkg.ccm.ImplementationDeployer;
import org.objectweb.ccm.descriptor.softpkg.ccm.SoftpkgDeployer;

/**
 * The homeplacement deployer context defines all the connection requires for
 * the installation of a home.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class HomeplacementDeployerContext 
       extends ChildDeployerContext 
    {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================    
    private HomeplacementBean homeplacement;
    private HomeplacementDeployInfo homeplacementInfo;
    private LinkedList componentInstantiationDeployers;
    private LinkedList registerwithhomefinderDeployers;
    private LinkedList registerwithnamingDeployers;
    private LinkedList registerwithtraderDeployers;
    private ImplementationDeployer implementationDeployer;
    private SoftpkgDeployer softpkgDeployer;
    private DestinationDeployer destinationDeployer;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    public HomeplacementDeployerContext() 
    {
        componentInstantiationDeployers = new LinkedList();
        registerwithhomefinderDeployers = new LinkedList();
        registerwithnamingDeployers = new LinkedList();
        registerwithtraderDeployers = new LinkedList();
    }
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public accessors methods.
    //
    // ==================================================================
    public void 
    connectRegisterwithhomefinderDeployer(RegisterwithhomefinderDeployer reg)
    {
        this.registerwithhomefinderDeployers.addLast(reg);
    }

    public RegisterwithhomefinderDeployer[] 
    getRegisterwithhomefinderDeployers() 
    {
        return (
            RegisterwithhomefinderDeployer[]) registerwithhomefinderDeployers
                .toArray(
            new RegisterwithhomefinderDeployer[registerwithhomefinderDeployers
                .size()]);
    }

    public void 
    connectRegisterwithtraderDeployer(RegisterwithtraderDeployer reg) 
    {
        this.registerwithtraderDeployers.addLast(reg);
    }

    public RegisterwithtraderDeployer[] 
    getRegisterwithtraderDeployers() 
    {
        return (
            RegisterwithtraderDeployer[]) registerwithtraderDeployers.toArray(
            new RegisterwithtraderDeployer[registerwithtraderDeployers.size()]);
    }

    public void 
    connectRegisterwithnamingDeployer(RegisterwithnamingDeployer reg) 
    {
        this.registerwithnamingDeployers.addLast(reg);
    }

    public RegisterwithnamingDeployer[] 
    getRegisterwithnamingDeployers() 
    {
        return (
            RegisterwithnamingDeployer[]) registerwithnamingDeployers.toArray(
            new RegisterwithnamingDeployer[registerwithnamingDeployers.size()]);
    }

    public void 
    connectHomeplacement(HomeplacementBean homeplacement) 
    {
        this.homeplacement = homeplacement;
        connectDeployerDescription(homeplacement);
    }

    public Homeplacement 
    getHomeplacement() 
    {
        return homeplacement;
    }

    public void 
    connectComponentInstantiationDeployer(ComponentinstantiationDeployer componentInstantiationDeployer) 
    {
        this.componentInstantiationDeployers.addLast(
            componentInstantiationDeployer);
    }

    public ComponentinstantiationDeployer[] 
    getComponentInstantiationDeployers() 
    {
        return (
            ComponentinstantiationDeployer[]) componentInstantiationDeployers
                .toArray(
            new ComponentinstantiationDeployer[componentInstantiationDeployers
                .size()]);
    }
    /**
     * @return
     */
    public ImplementationDeployer 
    getImplementationDeployer() 
    {
        return implementationDeployer;
    }

    /**
     * @param deployer
     */
    public void 
    connectImplementationDeployer(ImplementationDeployer deployer) 
    {
        implementationDeployer = deployer;
    }

    /**
     * @return
     */
    public SoftpkgDeployer 
    getSoftpkgDeployer() 
    {
        return softpkgDeployer;
    }

    /**
     * @param deployer
     */
    public void 
    connectSoftpkgDeployer(SoftpkgDeployer deployer) 
    {
        softpkgDeployer = deployer;
    }

    /**
        * @return
        */
    public DestinationDeployer 
    getDestinationDeployer()
    {
        return destinationDeployer;
    }

    /**
     * @param deployer
     */
    public void 
    connectDestinationDeployer(DestinationDeployer deployer) 
    {
        destinationDeployer = deployer;
    }

    /**
     * @return
     */
    public HomeplacementDeployInfo 
    getHomeplacementInfo() 
    {
        return homeplacementInfo;
    }

    /**
     * @param handler
     */
    public void 
    connectHomeplacementInfo(HomeplacementDeployInfo handler) 
    {
        homeplacementInfo = handler;
    }
    
    /**
     * This initialization method is used to check that all the services and 
     * information required to deploy are presents. If all is OK the deployer 
     * is put in inactive deployer state waiting for the deployment processing.
     * 
     * @exception InitializationError is thrown if a problem is detected 
     */
    public void 
    initialize() 
    throws InitializationError 
    {
        if (getHomeplacement() == null)
            getErrorManager().submitException(
                new InitializationError(
                    this,
                    "Connection on homeplacement bean is required"
                        + "check the handler code it must specify this connection"));
        if (getHomeplacement().getComponentfileref() == null
            || getHomeplacement().getComponentfileref().getIdref() == null
            || getHomeplacement().getComponentfileref().getIdref().length() == 0)
            getErrorManager().submitException(
                new InitializationError(
                    this,
                    "The homeplacement element must be fullfilled "
                        + "including its idref attribute, check the homeplacement element"
                        + "description\n."
                        + getStringifiedDeployerDescription()));
        if(getDestinationDeployer()==null)
            getErrorManager().submitException(
                new InitializationError(
                    this,
                    "The homeplacement requires a Destination to be installed "
                        + "check the homeplacement element description\n."
                        + getStringifiedDeployerDescription()));
                        
        getLifeCycleManager().stepEndedSuccessFully();
    }


}
