/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.registrar;


import org.objectweb.ccm.descriptor.componentassembly.Registerwithhomefinder;
/**
 * Specific deployer allowing to register/unregister a home
 * in the homefinder.
 *
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */

public class RegisterwithhomefinderDeployer 
       extends RegisterwithhomefinderDeployerContext
    {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    /* the home registration cookie*/
    private org.omg.Components.Cookie cookie;  
    /*registration home name*/
    private String registrationName;
    /*System.err.println prefix*/
    private String prefix=">> HomeFinder Registration >>";
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

    /**
     * Public method to register a home.
     *
     * @param descriptor descriptor containing CCD file path.
     * @param homeToRegister The home to register.  
     */
    public void
    register_home(org.objectweb.ccm.descriptor.softpkg.Descriptor descriptor,
                  org.omg.Components.CCMHome homeToRegister)
    {    
        Registerwithhomefinder rwhfi=getRegisterwithhomefinder();
        if(getRootDeployerContext().getHomeRegistration()==null)
            {
       
                System.err.println
                   (prefix+ "Cannot found HomeFinder reference, the HomeFinder service hasn't be started");
                System.err.println
                   (prefix+ "To start the HomeFinder service from console just execute finder_start script");
                System.err.println
                    (prefix+"The HomeFinder registration will be skipped");
         /*Skip the registration*/
         return ;
        }
        
        try{
            cookie=getHomeregistration()
                    .register_home(homeToRegister,
                                   getDescriptorDeployer().getHomeRepid(),
                                   getDescriptorDeployer().getComponentRepid(),
                                   rwhfi.getName());     
       }    
        catch(Exception e){
            System.err.println(prefix+"Problem occured during homefinder registration of"+ rwhfi.getName());
            System.err.println(e.getMessage());
        }  
        getLifeCycleManager().stepEndedSuccessFully();      
    }
    
    /**
     * Public method to unregister the home registered by this deployer instance.
     */
    public void
    unregister_home()
    {   
        try{           
            if(cookie!=null)
                getHomeregistration().unregister_home(cookie);    
        }
        catch(Exception e){            
            System.err.println(prefix+" Problem occured during unregistration of "+registrationName);
            System.err.println(e.getMessage());       
        }
        getLifeCycleManager().stepEndedSuccessFully();
    }
        

}
