/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.generator.common.lib;

// Package dependencies.

/**
 * This class manages creation of a directory path.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class FileManager
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     **/
    public FileManager()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================


    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

    /**
     * Create directories for a given path,
     * including any necessary but nonexistent parent directories.
     *
     * @param dir - A directory.
     **/
    public static void
    mkdir(java.io.File dir)
    throws GenerationException
    {
        if (dir.exists())
        {
            if (!dir.isDirectory())
            {
                String msg = new String(dir + " is not a directory!");
                throw new GenerationException(msg);
            }
        }
        else
        {
            dir.mkdirs();
        }
    }

    /**
     * Create directories for a given path,
     * including any necessary but nonexistent parent directories.
     *
     * @param dir_name - A pathname string.
     *
     * @return The corresponding File.
     **/
    public static java.io.File
    mkdir(String dir_name)
    throws GenerationException
    {
        java.io.File dir = new java.io.File(dir_name);
        mkdir(dir);
        return dir;
    }
}
