/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): .

====================================================================*/

package org.objectweb.corba.generator.idl.api;


/** To use ArrayList. */
import java.util.ArrayList;

/** To access all the AST Scope. */
import org.objectweb.corba.ast.api.Scope;

/** To access Generation Exception. */
import org.objectweb.corba.generator.common.lib.GenerationException;


public interface IDL3Generator
         extends org.objectweb.corba.generator.common.api.Generator
{
    /**
     * Initialize the generator.
     *
     * @param outputfile - The name where declarations will be generated.
     * @param imports - IR declarations to import.
     * @param app_name - The application's name.
     **/
    public void
    initialize( String outputfile,
                ArrayList imports,
                String app_name );

    /**
     * Generates contents of a scope.
     * name must be a valid declaration name.
     *
     * @param name - The name of the declaration to visit.
     *
     * @throws GenerationException
     **/
    public void
    generate(String name)
    throws GenerationException;

    /**
     * Generates contents of a scope.
     *
     * @param scope - The scope to visit.
     *
     * @throws GenerationException
     **/
    public void
    generate(Scope scope)
    throws GenerationException;

    /**
     * Generates contents of the root scope.
     * Declarations types must match to limited types.
     *
     * @param target_decl - The scope to visit.
     * @param limited_types - A logical combination of DeclarationKind.
     *
     * @throws GenerationException
     **/
    public void
    generate(Scope target_decl, long limited_types)
    throws GenerationException;
}
