/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.corba.generator.java.ast.lib;

import org.objectweb.corba.generator.java.ast.api.MethodObject;
import org.objectweb.corba.generator.java.ast.api.AbstractObject;
import org.objectweb.corba.generator.java.ast.api.StaticObject;
import org.objectweb.corba.generator.java.ast.api.AttributeObject;
import org.objectweb.corba.generator.java.ast.api.MethodContent;
import java.util.ArrayList;

/**
 * This is the representation for java Methodes.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 */

public class MethodObjectImpl
     extends ObjectBaseImpl
  implements MethodObject
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The delegation for an abstract */
    private AbstractObject abstract_delegate_;

    /** The delegation for a static */
    private StaticObject static_delegate_;

    /** The return type */
    private AttributeObject return_type_;

    /** Method parameters */
    private ArrayList parameters_;

    /** Method exceptions */
    private ArrayList exceptions_;

    /** Method implementation */
    private MethodContent impl_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    MethodObjectImpl()
    {
        // Init internal states
        super();
        abstract_delegate_ = new AbstractObjectImpl();
        static_delegate_ = new StaticObjectImpl();
        return_type_ = null;
        parameters_ = new ArrayList();
        exceptions_ = new ArrayList();
        impl_ = new MethodContentImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.objectweb.corba.generator.java.ast.api.AbstractObject
    //
    // ==================================================================

    /**
     * Set abstract this object.
     *
     * @param b - The value to set.
     */
    public void
    setAbstract(boolean b)
    {
        abstract_delegate_.setAbstract(b);
    }

    /**
     * Is this object abstract ?
     *
     * @return True if it is abstract, else false.
     */
    public boolean
    isAbstract()
    {
        return abstract_delegate_.isAbstract();
    }

    // ==================================================================
    //
    // Methods for org.objectweb.corba.generator.java.ast.api.StaticObject
    //
    // ==================================================================

    /**
     * Set static this object.
     *
     * @param b - The value to set.
     */
    public void
    setStatic(boolean b)
    {
        static_delegate_.setStatic(b);
    }

    /**
     * Is this object static ?
     *
     * @return True if it is static, else false.
     */
    public boolean
    isStatic()
    {
        return static_delegate_.isStatic();
    }

    // ==================================================================
    //
    // Methods for org.objectweb.corba.generator.java.ast.api.MethodObject
    //
    // ==================================================================

    /**
     * Set its return type.
     *
     * @param type - The type to set.
     */
    public void
    setReturnType(String type)
    {
        return_type_ = new AttributeObjectImpl();
        return_type_.setType(type);
    }

    /**
     * Obtain its name.
     *
     * @return Its name.
     */
    public AttributeObject
    getReturnType()
    {
        return return_type_;
    }

    /**
     * Add a parameter.
     *
     * @param param - The parameter to add.
     */
    public void
    addParameter(AttributeObject param)
    {
        parameters_.add(param);
    }

    /**
     * Obtain all parameters.
     *
     * @return A list of parameters.
     */
    public ArrayList
    getParameters()
    {
        return parameters_;
    }

    /**
     * Obtain its implementation.
     *
     * @return Its implementation.
     */
    public MethodContent
    getImpl()
    {
        return impl_;
    }

    /**
     * Add an exception.
     *
     * @param ex - The exception to add.
     */
    public void
    addException(String ex)
    {
        exceptions_.add(ex);
    }

    /**
     * Obtain all exceptions.
     *
     * @return A list of exceptions.
     */
    public ArrayList
    getExceptions()
    {
        return exceptions_;
    }
}
