/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.corba.generator.translator.ast2idl.lib;

import org.objectweb.corba.ast.api.TypeKind;
import org.objectweb.corba.ast.api.Declaration;

/**
 * Utility interface to translate IDL types in string format.
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.5
 */

public class AST_IDLTranslator
  implements org.objectweb.corba.generator.translator.ast2idl.api.AST_IDLTranslator
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     * Java keywords table.
     */
    protected java.util.Set keywords_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     *  The default constructor.
     **/
    public
    AST_IDLTranslator()
    {
        keywords_ = new java.util.HashSet();
        keywords_.add("abstract");
        keywords_.add("any");
        keywords_.add("attribute");
        keywords_.add("boolean");
        keywords_.add("case");
        keywords_.add("char");
        keywords_.add("const");
        keywords_.add("context");
        keywords_.add("custom");
        keywords_.add("default");
        keywords_.add("double");
        keywords_.add("exception");
        keywords_.add("enum");
        keywords_.add("factory");
        keywords_.add("FALSE");
        keywords_.add("fixed");
        keywords_.add("float");
        keywords_.add("in");
        keywords_.add("inout");
        keywords_.add("interface");
        keywords_.add("local");
        keywords_.add("long");
        keywords_.add("module");
        keywords_.add("native");
        keywords_.add("Object");
        keywords_.add("octet");
        keywords_.add("oneway");
        keywords_.add("out");
        keywords_.add("private");
        keywords_.add("public");
        keywords_.add("raises");
        keywords_.add("readonly");
        keywords_.add("sequence");
        keywords_.add("short");
        keywords_.add("string");
        keywords_.add("struct");
        keywords_.add("supports");
        keywords_.add("switch");
        keywords_.add("TRUE");
        keywords_.add("truncatable");
        keywords_.add("typedef");
        keywords_.add("unsigned");
        keywords_.add("union");
        keywords_.add("ValueBase");
        keywords_.add("valuetype");
        keywords_.add("void");
        keywords_.add("wchar");
        keywords_.add("wstring");
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Translate a type into its idl format.
     *
     * @param typeref - The type reference to translate.
     *
     * @return The resulting string.
     **/
    public String
    toIDL(org.objectweb.corba.ast.api.TypeRef typeref)
    {
        TypeKind tk = typeref.getTypeKind();

        if (tk == TypeKind.tk_null)
            return "null";
        else if (tk == TypeKind.tk_void)
            return "void";
        else if (tk ==  TypeKind.tk_short)
            return "short";
        else if (tk == TypeKind.tk_ushort)
            return "unsigned short";
        else if (tk == TypeKind.tk_long)
            return "long";
        else if (tk == TypeKind.tk_ulong)
            return "unsigned long";
        else if (tk == TypeKind.tk_longlong)
            return "long long";
        else if (tk == TypeKind.tk_ulonglong)
            return "unsigned long long";
        else if (tk == TypeKind.tk_float)
            return "float";
        else if (tk == TypeKind.tk_double)
            return "double";
        else if (tk == TypeKind.tk_longdouble)
            return "long double";
        else if (tk == TypeKind.tk_boolean)
            return "boolean";
        else if (tk == TypeKind.tk_char)
            return "char";
        else if (tk == TypeKind.tk_wchar)
            return "wchar";
        else if (tk == TypeKind.tk_octet)
            return "octet";
        else if (tk == TypeKind.tk_any)
            return "any";
        else if (tk == TypeKind.tk_Principal)
            return "::CORBA::Principal";
        else if (tk == TypeKind.tk_TypeCode)
            return "::CORBA::TypeCode";

        else if (tk == TypeKind.tk_objref)
            return "Object";

        else if ( (tk == TypeKind.tk_struct) ||
                  (tk == TypeKind.tk_union) ||
                  (tk == TypeKind.tk_enum) ||
                  (tk == TypeKind.tk_exception) ||
                  (tk == TypeKind.tk_alias) ||
                  (tk == TypeKind.tk_value_box) ||
                  (tk == TypeKind.tk_value) ||
                  (tk == TypeKind.tk_native) ||
                  (tk == TypeKind.tk_abstract_interface) ||
                  (tk == TypeKind.tk_local_interface) ||
                  (tk == TypeKind.tk_interface) ||
                  (tk == TypeKind.tk_component) ||
                  (tk == TypeKind.tk_event) ||
                  (tk == TypeKind.tk_home) )
        {
                Declaration decl = (Declaration)typeref;
                return getAbsoluteName(decl);
        }
        else if (tk == TypeKind.tk_string)
        {
                int l = ((org.objectweb.corba.ast.api.TypeRefWithLength)typeref).getLength();
                if (l==0)
                    return "string";
                return "string<"+l+">";
        }
        else if (tk == TypeKind.tk_wstring)
        {
                int l = ((org.objectweb.corba.ast.api.TypeRefWithLength)typeref).getLength();
                if (l==0)
                    return "wstring";
                return "wstring<"+l+">";
        }
        else if (tk == TypeKind.tk_sequence)
        {
                org.objectweb.corba.ast.api.TypeRefWithContent type = (org.objectweb.corba.ast.api.TypeRefWithContent)typeref;
                int l = type.getLength();
                String str = toIDL(type.getContentType());
                if (l==0)
                    return "sequence<"+str+">";
                return "sequence<"+str+","+l+">";
        }
        else if (tk == TypeKind.tk_array)
        {
                org.objectweb.corba.ast.api.TypeRefWithContent type = (org.objectweb.corba.ast.api.TypeRefWithContent)typeref;
                String str = toIDL(type.getContentType());
                return str;
        }
        else if (tk == TypeKind.tk_fixed)
        {
                org.objectweb.corba.ast.api.FixedTypeRef type = (org.objectweb.corba.ast.api.FixedTypeRef)typeref;
                return "fixed<"+type.getDigits()+","+type.getScale()+">";
        }
        else
                return "";
    }

    /**
     * Add postfix to the IDL name if typeref is an array.
     *
     * @param typeref - The array to translate.
     *
     * @return The resulting string.
     **/
    public String
    toIDLPostfix(org.objectweb.corba.ast.api.TypeRef typeref)
    {
        String res = "";

        if ( typeref.getTypeKind() == TypeKind.tk_array )
        {
                org.objectweb.corba.ast.api.TypeRefWithContent type = null;
                type = (org.objectweb.corba.ast.api.TypeRefWithContent)typeref;
                res = toIDLPostfix(type.getContentType())+"["+type.getLength()+"]";
        }
        return res;
    }

    /**
     * Get the string value of an Any.
     *
     * @param anyvalue - The Any to translate.
     * @param typeref - The value's type.
     *
     * @return The resulting string.
     **/
    public String
    toString(org.objectweb.corba.ast.api.AnyValue anyvalue,
             org.objectweb.corba.ast.api.TypeRef typeref)
    {
        //
        //  TODO : add escaped sequencesto strings, eg '\n' => "\\n"
        //
        if (anyvalue.isInteger())
            return java.lang.String.valueOf(anyvalue.getAsInteger());
        else if (anyvalue.isString())
            return "\""+anyvalue.getAsString()+"\"";
        else if (anyvalue.isWString())
            return "L\""+anyvalue.getAsWString()+"\"";
        else if (anyvalue.isChar())
            return "\'"+java.lang.String.valueOf(anyvalue.getAsChar())+"\'";
        else if (anyvalue.isWChar())
            return "L\'"+java.lang.String.valueOf(anyvalue.getAsWChar())+"\'";
        else if (anyvalue.isFloating())
            return java.lang.String.valueOf(anyvalue.getAsFloating());
        else if (anyvalue.isFixed())
            return anyvalue.getAsFixed();
        else if (anyvalue.isBoolean())
        {
            if (anyvalue.getAsBoolean())
                return "TRUE";
            else
                return "FALSE";
        }
        else if (anyvalue.isEnum())
        {
            // build enum member absolute name
            String abs_name = getAbsoluteName( ((org.objectweb.corba.ast.api.EnumDecl)typeref).getParent() );
            return abs_name + "::" + anyvalue.getAsEnum();
        }
        else
            return "";
    }

    /**
     * Check if id is an idl keyword.
     *
     * @param id - The string to check
     *
     * @return A valid string.
     */
    public java.lang.String
    checkKeywords(String id)
    {
        if (keywords_.contains( id.toLowerCase() ))
           return "_"+id;

        return id;
    }

    /**
     * Get a valid absolute name from a declaration,
     * i.e. check keywords.
     *
     * @param decl - The declaration to get the name.
     *
     * @return A valid absolute name.
     */
    public String
    getAbsoluteName(Declaration decl)
    {
        String result = "";

        if (decl.getParent() != null)
            result = getAbsoluteName( decl.getParent() );

        if ( (decl.getName() != null) &&
             !(decl.getName().equals("")) )
            result = result + "::" + checkKeywords(decl.getName());

        return result;
    }

}
