/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/

package org.objectweb.openccm.browser.CosNaming;

import org.objectweb.util.browser.api.TreeView;
import org.objectweb.util.browser.api.Table;
import org.objectweb.util.browser.core.naming.DefaultName;
import org.objectweb.util.browser.core.naming.DefaultEntry;
import org.objectweb.openccm.browser.CORBA.ConsoleFactory;
import org.objectweb.openccm.browser.util.ior.IorPrinter;
import java.util.List;
import java.util.Vector;
import java.util.Iterator;

import org.omg.CosNaming.NamingContextExt;

/**
 * Displays objects bound into the given <code>CosNaming</code> object.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class CosNamingPanel 
    implements Table {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================
    
    //==================================================================
    //
    // No constructor.
    //
    //==================================================================

    //==================================================================
    //
    // Internal methods.
    //
    //==================================================================  

    //==================================================================
    //
    // Public methods for CosNamingPanel interface.
    //
    //==================================================================

    /**
     * Overriding methods
     * @see org.objectweb.util.browser.api.Table#getHeaders()
     */
    public String[] getHeaders(TreeView treeView) {
        return new String[]{"ID","Kind","Value"};
    }
    
    /**
     * Overriding methods
     * @see org.objectweb.util.browser.api.Table#getRows(org.objectweb.util.browser.api.TreeView)
     */
    public Object[][] getRows(TreeView treeView) {
        NamingContextExt naming = (NamingContextExt)treeView.getSelectedObject();
        org.omg.CORBA.ORB orb = org.objectweb.ccm.CORBA.TheORB.getORB();
        IorPrinter iorPrinter = null;
        List l = new Vector();
        try{
            org.omg.CosNaming.BindingListHolder bindingList = new org.omg.CosNaming.BindingListHolder();
            org.omg.CosNaming.BindingIteratorHolder bindingIterator = new org.omg.CosNaming.BindingIteratorHolder();
            naming.list(0, bindingList, bindingIterator);
            org.omg.CosNaming.BindingHolder binding = new org.omg.CosNaming.BindingHolder();
            if(bindingIterator.value!=null){
                while(bindingIterator.value.next_one(binding)){
                    try{
                        org.omg.CosNaming.NameComponent[] nameComponent = binding.value.binding_name;
                        org.omg.CORBA.Object object = naming.resolve(nameComponent);
                        iorPrinter = new IorPrinter(orb.object_to_string(object));

                        String id="",kind="";
                        id = nameComponent[nameComponent.length-1].id;
                        kind = nameComponent[nameComponent.length-1].kind;

                        l.add(new Object[]{id,kind,new DefaultEntry(object,new DefaultName(iorPrinter.getTypeId()),null)});
                    } catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
                        System.out.println(getClass().getName() + " : Object Not Found Exception !");
                    } catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e) {
                        System.out.println(getClass().getName() + " : CannotProceed Exception");
                    } catch (org.omg.CosNaming.NamingContextPackage.InvalidName e) {
                        System.out.println(getClass().getName() + " : InvalidName Exception");
                    }
                }
            }
        } catch(Exception e) {
            ConsoleFactory.getDebugConsole().add(e.getMessage()+ "\n");
        }

        Object[][] contenu = new Object[l.size()][2];
        Iterator it = l.iterator();
        int cpt = 0;
        while(it.hasNext()){
            contenu[cpt] = (Object[])it.next();
            cpt++;
        }

        return contenu;
    }
}
