/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser.CosNaming;

/** The administration console's API imports */
import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.core.naming.AbstractContext;
import org.objectweb.util.browser.core.api.Name;
import org.omg.CosNaming.NamingContextExt;
import org.objectweb.openccm.browser.CORBA.ConsoleFactory;

/** The Java API's imports */
import java.util.StringTokenizer;
import java.util.List;
import java.util.Vector;

/**
 * Basic implementation of the Context interface
 * for managing NamingContext.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class NameServiceContext 
    extends AbstractContext {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The Naming context. */
    private NamingContextExt object_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // No internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for Context interface.
    //
    // ==================================================================

    public Entry[] getEntries() {
        List l = new Vector();
        try{
            org.omg.CosNaming.BindingListHolder bindingList = new org.omg.CosNaming.BindingListHolder();
            org.omg.CosNaming.BindingIteratorHolder bindingIterator = new org.omg.CosNaming.BindingIteratorHolder();
            object_.list(0, bindingList, bindingIterator);
            org.omg.CosNaming.BindingHolder binding = new org.omg.CosNaming.BindingHolder();
            if(bindingIterator.value!=null){
                while(bindingIterator.value.next_one(binding)){
                    try{
                        l.add(createEntry(object_.resolve(binding.value.binding_name), object_.to_string(binding.value.binding_name)));
                    } catch (org.omg.CosNaming.NamingContextPackage.NotFound e) {
                        System.out.println(getClass().getName() + " : Object Not Found Exception !");
                    } catch (org.omg.CosNaming.NamingContextPackage.CannotProceed e) {
                        System.out.println(getClass().getName() + " : CannotProceed Exception");
                    } catch (org.omg.CosNaming.NamingContextPackage.InvalidName e) {
                        System.out.println(getClass().getName() + " : InvalidName Exception");
                    }
                }
            }
        } catch(Exception e) {
            ConsoleFactory.getDebugConsole().add(e.getMessage()+ "\n");
        }
        return (Entry[])l.toArray(new Entry[0]);
    }

    // ==================================================================
    //
    // Additional public methods for Wrapper interface.
    //
    // ==================================================================

    /**
     * Fix the wrapped object
     *
     * @param object The object to wrap
     *
     */
    public void setWrapped(Object object) {
        object_ = (NamingContextExt) object;
    }

    /**
     * Gets the wrapped object
     *
     * @return The wrapped object
     *
     */
    public Object getWrapped() {
        return object_;
    }

}
