/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/

package org.objectweb.openccm.browser.DCI;

/** The Browser's imports */
import java.util.Vector;

import org.objectweb.util.browser.api.TreeView;
import org.objectweb.util.browser.api.Table;
import org.objectweb.util.browser.core.naming.DefaultName;
import org.objectweb.util.browser.core.naming.DefaultEntry;

/** The CCM's imports */
import org.ist.coach.DCI.AssemblyManager;
import org.ist.coach.DCI.DCIDeployment;
import org.ist.coach.DCI.DCIManager;
import org.ist.coach.DCI.DCIInformation;
import org.ist.coach.DCI.UnknownAssembly;

/**
 * Displays the installed assemblies for the given DCIManager.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class DCIInstalledAssembliesTable
  implements Table 
{

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================
    
    //==================================================================
    //
    // No constructor.
    //
    //==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Provides a name depending on the AssemblyManager id.
     */
    protected String getName(AssemblyManager assemblyManager){
        String returnedValue = "Assembly ";
        String assemblyId = assemblyManager.ass_id();
        if(assemblyId.indexOf(":")!=-1)
            returnedValue = assemblyId.substring(0,assemblyId.indexOf(":"));
        return returnedValue;
    }  

    /**
     * Provides a name depending on the AssemblyManager id.
     */
    protected String getName(String assemblyUUID){
        String returnedValue = "Assembly ";
        if(assemblyUUID!=null && assemblyUUID.indexOf(":")!=-1)
            returnedValue = assemblyUUID.substring(0,assemblyUUID.indexOf(":")) + " Assembly"; 
        return returnedValue;
    }
    
    //==================================================================
    //
    // Public methods for Table interface.
    //
    //==================================================================

    /**
     * Overriding methods
     * @see org.objectweb.util.browser.api.Table#getHeaders()
     */
    public String[] getHeaders(TreeView treeView) {
        return new String[]{"Model","Assemblies"};
    }
    
    /**
     * Overriding methods
     * @see org.objectweb.util.browser.api.Table#getRows(org.objectweb.util.browser.api.TreeView)
     */
    public Object[][] getRows(TreeView treeView) {
        DCIManager DCIManager_ = (DCIManager)treeView.getSelectedObject();
        DCIInformation DCIInfo = DCIManager_.provide_dci_information();
        DCIDeployment DCIDeploy = DCIManager_.provide_dci_deployment();        
        String[] assemblyUUIDs = DCIDeploy.get_assemblies();
        
        Vector values = new Vector();
        for (int i = 0; i < assemblyUUIDs.length; i++){
            // Calculating the assemblies for this model.
            try{
                String[] instanceUUID = DCIDeploy.get_assembly_instances(assemblyUUIDs[i]);
                if(instanceUUID.length>0){
                    boolean isFirst = true;
                    for (int j = 0; j < instanceUUID.length; j++) {
                        AssemblyManager assemblyManager = DCIDeploy.get_assembly_manager(instanceUUID[j]);
                        AssemblyManagerWrapperWithDCIDeployment am = new AssemblyManagerWrapperWithDCIDeployment();
                        am.setAssemblyManager(assemblyManager);
                        am.setDCIDeployment(DCIDeploy);
                        am.setUUID(instanceUUID[j]);
                        if(isFirst){
                            isFirst = false;
                            values.add(new Object[]{new DefaultEntry(new AssemblyUUIDWrapper(assemblyUUIDs[i],DCIDeploy), new DefaultName(getName(assemblyUUIDs[i]) + " (" + (i+1) + ")" )), new DefaultEntry(am, new DefaultName(getName(assemblyManager) + " (" + (j+1) + ")"))});
                        } else {
                            values.add(new Object[]{null,new DefaultEntry(am, new DefaultName(getName(assemblyManager) + " (" + (j+1) + ")"))});
                        }
                    }
                } else {
                    values.add(new Object[]{new DefaultEntry(new AssemblyUUIDWrapper(assemblyUUIDs[i],DCIDeploy), new DefaultName(getName(assemblyUUIDs[i]) + " (" + (i+1) + ")")),null});
                }
            } catch (UnknownAssembly e) {
                e.printStackTrace();
            }
        }
             
        Object[][] contenu = new Object[values.size()][2];
        for(int i=0 ; i<values.size() ; i++){
            contenu[i] = (Object[])values.get(i);
        }
        
        return contenu;
    }
}
