/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser.Deployment;

/** The administration console's API imports */
import org.objectweb.util.browser.api.MenuItemTreeView;
import org.objectweb.util.browser.api.MenuItem;
import org.objectweb.util.browser.api.TreeView;
import org.objectweb.util.browser.gui.api.DialogAction;
import org.objectweb.util.browser.gui.api.DialogBox;
import org.objectweb.util.browser.gui.lib.FileChooserBox;
import org.objectweb.util.browser.gui.lib.DefaultDialogBox;
import org.objectweb.openccm.browser.menu.JFileChooserSingleton;
import org.objectweb.openccm.browser.menu.OpenCCMBrowserConstants;

/** The Java API imports */
import java.net.URL;

/** The OpenCCM platform imports */
import org.objectweb.ccm.deploytool.MicroServerHttp;
import org.omg.Components.Deployment.AssemblyFactory;

/**
 * The <b>create</b> operation creates an Assembly object on the host
 * on which the AssemblyFactory is located. It takes a string denoting
 * the location of an assembly. The URL can be found on the local file
 * system or can be defined as a distant URL.
 * (section 6.9.3 of the CORBA Components Model specification)
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class CreateAssembly
  implements MenuItem, DialogAction {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    /** The fileChooserBox containing the URL of the archive. */
    protected FileChooserBox archive_ = null;

    /** The AssemblyFactory on which creates the Assembly. */
    protected AssemblyFactory assemblyFactory_ = null;

    //==================================================================
    //
    // No constructor.
    //
    //==================================================================

    //==================================================================
    //
    // Internal methods.
    //
    //==================================================================

    protected AssemblyFactory getAssemblyFactory(TreeView treeView){
        return (AssemblyFactory)treeView.getSelectedObject();
    }
    
    //==================================================================
    //
    // Public methods for MenuItem interface.
    //
    //==================================================================

    public int getStatus(TreeView arg0){
        return MenuItem.ENABLED_STATUS;
    }

    public void actionPerformed(MenuItemTreeView e) throws Exception {
        assemblyFactory_ = getAssemblyFactory(e);
        
        DialogBox dialog = new DefaultDialogBox("Create an assembly");
        archive_ = new FileChooserBox("Archive",JFileChooserSingleton.getInstance(OpenCCMBrowserConstants.ASSEMBLY_ARCHIVE_FILE_CHOOSER),true,true);
        dialog.addElementBox(archive_);
        dialog.setValidateAction(this);
        dialog.show();
    }
    
    //==================================================================
    //
    // Public methods for DialogAction interface.
    //
    //==================================================================

    /**
     * Creates the Assembly corresponding to the param archive.
     */
    public void executeAction() throws Exception {
        URL theURL = null;
        if(archive_.getFile()!=null){
            // The local file
            // Start the HTTP Server
            java.net.ServerSocket s = new java.net.ServerSocket(0);
            (new Thread(new MicroServerHttp(s))).start();
            theURL= new URL("http://" + java.net.InetAddress.getLocalHost().getHostAddress() + ':' + s.getLocalPort() + '/' + archive_.getFile().getAbsolutePath());
        } else {
            if(archive_.getURL()!=null){
                // A valid URL has been filled
                theURL = archive_.getURL();
            }
        }
        if(theURL!=null){
            assemblyFactory_.create_assembly(theURL.toString()); 
        }
    }
}