/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser.Deployment;

import org.objectweb.util.browser.api.MenuItemTreeView;
import org.objectweb.util.browser.api.MenuItem;
import org.objectweb.util.browser.api.TreeView;
import javax.swing.JOptionPane;

import org.omg.Components.Deployment.Assembly;

/**
 * The <b>tear_down</b> operation removes all connections between components and
 * destroys all components, homes, containers and component servers.
 * (section 6.9.4 of the CORBA Components Model specification)
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class TearDown
  implements MenuItem {

    protected Assembly getAssembly(TreeView e){
        return (Assembly)e.getSelectedObject();
    }

    public int getStatus(TreeView e){
        Assembly assembly = getAssembly(e);
        org.omg.Components.Deployment.AssemblyState state = assembly.get_state();
        if(state.equals(org.omg.Components.Deployment.AssemblyState.INACTIVE))
            return MenuItem.DISABLED_STATUS;
        else
            return MenuItem.ENABLED_STATUS;
    }

    public void actionPerformed(MenuItemTreeView e) throws Exception {
        if(JOptionPane.showConfirmDialog(null,"Do you want to tear this assembly down ?", "Tear down action", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE)==0){
            Assembly assembly = getAssembly(e);
            assembly.tear_down();        
        }
    }

}