/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser;

/** The administration console's API*/
import org.objectweb.util.browser.api.Tree;
import org.objectweb.util.browser.core.api.ContextContainer;
import org.objectweb.util.browser.core.api.TreeConfiguration;
import org.objectweb.util.browser.core.naming.DefaultContextContainer;
import org.objectweb.util.browser.core.common.DefaultTreePanel;
import org.objectweb.util.browser.core.common.DefaultViewPanel;
import org.objectweb.util.browser.core.common.DefaultStatusBar;
import org.objectweb.util.browser.core.common.DynamicTree;
import org.objectweb.util.browser.core.api.StatusBar;

import org.objectweb.openccm.browser.menu.Menu;
import org.objectweb.openccm.browser.menu.TreeDialogSingleton;

import org.objectweb.ccm.CORBA.TheORB;
import org.objectweb.openccm.browser.CORBA.ConsoleFactory;

/** The java API's imports */
import java.io.File;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.JFrame;
import javax.swing.ImageIcon;
import java.net.URL;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.BorderFactory;

/**
 * This class defines the main frame
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class Main extends JFrame {

    /** The used tree. */
    protected DynamicTree tree_;

    /**
     * Populates the tree
     */
    protected void populateTree(Tree tree) {

        // Adds the name service into the initial context
        try {
            org.omg.CORBA.Object o = TheORB.getORB().resolve_initial_references("NameService");
            tree.addEntry("NameService", o, 2);
        } catch (org.omg.CORBA.ORBPackage.InvalidName e1) {
            // NS is not started
            e1.printStackTrace();
        } catch (org.omg.CORBA.TRANSIENT e1) {
            e1.printStackTrace();
	        // Ignore (throws when the NameService is not started)
	    }

        // Adds the interface repository into the initial context
        try {
            org.omg.CORBA.Object o = TheORB.getORB().resolve_initial_references("InterfaceRepository");
            tree.addEntry("InterfaceRepository", o);
        } catch (org.omg.CORBA.ORBPackage.InvalidName e1) {
            // IR is not started
        }

        // Adds the trading service into the initial context
        try {
            org.omg.CORBA.Object o = TheORB.getORB().resolve_initial_references("TradingService");
            tree.addEntry("TradingService", o);
        } catch (org.omg.CORBA.ORBPackage.InvalidName e1) {
            // Trader is not started
        } catch (org.omg.CORBA.TRANSIENT e1) {
            // Ignore (throws when the TradingService is not started)
        }

    }

    protected void initializeTreeDialog(TreeConfiguration treeConfiguration) {
        DynamicTree tree = new DynamicTree();
        tree.setNewBrowserProperty(treeConfiguration.getBrowserProperty());
        tree.setNewContextProperty(treeConfiguration.getContextProperty());

        // Adds the name service into the initial context
        try {
            org.omg.CORBA.Object o = TheORB.getORB().resolve_initial_references("NameService");
            tree.addEntry("NameService", o, 2);
        } catch (org.omg.CORBA.ORBPackage.InvalidName e1) {
            // Trader is not started
        } catch (org.omg.CORBA.TRANSIENT e1) {
            // Ignore (throws when the TradingService is not started)
        }

        // Adds the trading service into the initial context
        try {
            org.omg.CORBA.Object o = TheORB.getORB().resolve_initial_references("TradingService");
            tree.addEntry("TradingService", o);
        } catch (org.omg.CORBA.ORBPackage.InvalidName e1) {
            // NS is not strated
        } catch (org.omg.CORBA.TRANSIENT e1) {
	        // Ignore (throws when the NameService is not started)
	    }

        // Adds the dci manager into the initial context
        try {
            org.omg.CORBA.Object o = TheORB.getORB().resolve_initial_references("DCI");
            tree.addEntry("DCI Manager", o, 1);
        } catch (org.omg.CORBA.ORBPackage.InvalidName e1) {
            // Trader is not started
        } catch (org.omg.CORBA.TRANSIENT e1) {
            // Ignore (throws when the TradingService is not started)
        }

        TreeDialogSingleton.setInstance(tree);
    }

    /** Returns an ImageIcon, or null if the path is invalid. */
    protected static ImageIcon createIcon(String imageName) {
        if (imageName != null) {
            URL urlFile = null;
            urlFile = Thread.currentThread().getContextClassLoader().getResource(imageName);            
            if(urlFile==null){            
				try {
					urlFile = new URL(imageName);
				} catch (java.net.MalformedURLException e) {
                    System.out.println(imageName + ": Malformed URL !");
				}
            }
            if(urlFile!=null){
                return new ImageIcon(urlFile);
            }
        }
        return null;
    }

    /** 
     * Initializes the OpenCCM Browser frame
     */
    protected void init(String contextPropertyFile, String[] browserPropertyFile){
        DefaultViewPanel viewPanel = new DefaultViewPanel();

        tree_ = new DynamicTree();

        //setIconImage(createIcon("icons/tools.png").getImage());

        // Creates the menu
        Menu menu = new Menu(tree_);
        
        tree_.setNewContextProperty(contextPropertyFile);
        tree_.setNewBrowserProperty(browserPropertyFile);
        tree_.setTargetPanel(viewPanel);
        tree_.setMultipleRolesEnabled(true);
        tree_.setCurrentRole(new String[]{"user view","Deployment administrator","Components administrator","CORBA Administrator","CosNaming administrator"});
        tree_.setJMenuBar(menu.getMenuBar());
        tree_.setJToolBar(menu.getToolBar());
        populateTree((Tree)tree_);
        
        // Creates the StatusBar
        StatusBar statusBar = new DefaultStatusBar();
        tree_.setStatusBar(statusBar);

        // Initializes the tree using in JDialog Boxes,
        initializeTreeDialog((TreeConfiguration) tree_);

        setJMenuBar(menu.getMenuBar());

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(menu.getToolBar(),BorderLayout.NORTH);
        getContentPane().add(new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true, new DefaultTreePanel(tree_), viewPanel), BorderLayout.CENTER);
        getContentPane().add((DefaultStatusBar)statusBar,BorderLayout.SOUTH);

        //setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        addWindowListener(new OpenCCMWindowListener());
        pack();

        // Centers the frame
        Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((screenSize.width - getWidth()) / 2, (screenSize.height - getHeight()) / 2);

        setVisible(true);        
    }

    /**
     * Default constructor
     */
    public Main(String contextPropertyFile, String[] browserPropertyFile) {

        super("OpenCCM Browser");
        init(contextPropertyFile, browserPropertyFile);

    }

    /**
     * Class to manage the window actions
     */
    protected class OpenCCMWindowListener extends WindowAdapter{
        
        public void windowClosing(WindowEvent e) {
            tree_.close();
            ConsoleFactory.getDebugConsole().add("Shutting the ORB down\n");
            TheORB.shutdown(false);
            ConsoleFactory.getDebugConsole().add("Destroying the ORB\n");
            TheORB.destroy();
            System.exit(0);
        }
        
    }

}
