/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser;

/** The administration console's API*/
import org.objectweb.util.browser.api.Tree;
import org.objectweb.util.browser.core.api.ContextContainer;
import org.objectweb.util.browser.core.api.TreeConfiguration;
import org.objectweb.util.browser.core.naming.DefaultContextContainer;
import org.objectweb.util.browser.core.common.DefaultTreePanel;
import org.objectweb.util.browser.core.common.DefaultViewPanel;
import org.objectweb.util.browser.core.common.DefaultStatusBar;
import org.objectweb.util.browser.core.common.DynamicTree;
import org.objectweb.util.browser.core.api.StatusBar;

import org.objectweb.openccm.browser.menu.Menu;
import org.objectweb.openccm.browser.menu.TreeDialogSingleton;

/** The java API's imports */
import java.io.File;
import java.awt.Dimension;
import java.awt.BorderLayout;
import javax.swing.JFrame;
import javax.swing.ImageIcon;
import java.net.URL;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.BorderFactory;
import javax.swing.JTabbedPane;

/**
 * This class defines the main frame of the OpenCCM Browser designed for PDA
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class MainPDA 
     extends Main 
{

    /** 
     * Initializes the OpenCCM Browser frame
     */
    protected void init(String contextPropertyFile, String[] browserPropertyFile){

        DefaultViewPanel viewPanel = new DefaultViewPanel();

        DynamicTree tree = new DynamicTree();

        JTabbedPane tabbedPane = new JTabbedPane();

        //setIconImage(createIcon("icons/tools.png").getImage());

        // Creates the menu
        Menu menu = new Menu(tree);
        
        tree.setNewContextProperty(contextPropertyFile);
        tree.setNewBrowserProperty(browserPropertyFile);
        tree.setTargetPanel(viewPanel);
        tree.setMultipleRolesEnabled(true);
        tree.setCurrentRole(new String[]{"user view","Deployment administrator","Components administrator","CORBA Administrator","CosNaming administrator"});
        tree.setJMenuBar(menu.getMenuBar());
        populateTree((Tree)tree);

        // Creates the StatusBar
        StatusBar statusBar = new DefaultStatusBar();
        tree.setStatusBar(statusBar);
                
        // Initializes the tree using in JDialog Boxes,
        initializeTreeDialog((TreeConfiguration) tree);

        setJMenuBar(menu.getMenuBar());

        getContentPane().setLayout(new BorderLayout());
        
        tabbedPane.addTab("Tree", new DefaultTreePanel(tree));
        tabbedPane.addTab("View", viewPanel);
        getContentPane().add(tabbedPane, BorderLayout.CENTER);
        getContentPane().add((DefaultStatusBar)statusBar,BorderLayout.SOUTH);

        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        pack();

        // Centers the frame
        Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((screenSize.width - getWidth()) / 2, (screenSize.height - getHeight()) / 2);

        setVisible(true);

    }

    /**
     * Default constructor
     */
    public MainPDA(String contextPropertyFile, String[] browserPropertyFile) {
        super(contextPropertyFile, browserPropertyFile);
    }
    
}
