/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.services.pss.jdo.api;


/**
 * This interface describe a JDO Session.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public interface StorageHomeBase
         extends org.objectweb.openccm.services.pss.common.api.StorageHomeBase
{
    /**
     * Must be called before create method invokation.
     */
    public void
    before_create();

    /**
     * Must be called after create method invokation.
     *
     * @param st - The storage type to make persistent.
     */
    public void
    after_create(org.objectweb.openccm.services.pss.jdo.api.StorageObject st);

    /**
     * Create a pid for a storage type.
     *
     * @param short_pid - The short_pid of the storage type to encode.
     *
     * @return The storage type PID.
     */
    public byte[]
    create_pid(String short_pid);

    /**
     * Must be called after create method invokation.
     *
     * @param fields_names - Name of fields contained in the key.
     * @param fields_types - Type of fields contained in the key.
     * @param key_values - Value of fields contained in the key.
     *
     * @return The storage object matching to key values, else null.
     *
     * @throws org.omg.CosPersistentState.NotFound if there is no matching storage object.
     */
    public Object
    find_by_key(String[] fields_names,
                String[] fields_types,
                java.lang.Object[] key_values)
    throws org.omg.CosPersistentState.NotFound;
}
