/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s):  Jrme Moroy__________________________.

===========================================================================*/

package org.objectweb.util.browser.naming.api;

/**
 * Entries are non-mutable structures representing entries in contexts.
 *
 * An Entry is composed of a <value,name,context>,
 * where value is an Object, name is a Name, and
 * context is the owner Context.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @version 0.1
 *
 * Note: This is strongly inspired from the Jonathan naming context framework
 * initially developped by Fabien Delpiano and Bruno Dumant.
 *
 * Here is a simplification of the Element interface.
 */

public interface Entry {
	
    /**
     * The standard name of this interface to use
     * in the context of the Fractal framework.
     */
    static final String ENTRY = Entry.class.getName();

    /**
     * Gets the value contained in the target entry.
     *
     * This method does not update the value contained in the target entry.
     *
     * @return The value contained in the target entry.
     *
     * @postcondition return == getOWValue();
     */
    Object getOWValue();

    /**
     * Gets the name of the target entry, in the context it belongs to.
     *
     * This method does not update the name of the target entry.
     *
     * @return The name of the target entry, in the context it belongs to.
     *
     * @postcondition return == getOWName();
     */
    Name getOWName();

    /**
     * Gets the context containing the target entry,
     * if any, null otherwise.
     *
     * This method does not update the context containing
     * the target entry.
     *
     * @return The context containing the target entry,
     *         if any, null otherwise..
     *
     * @postcondition return == getOWContext();
     */
    Context getOWContext();

}
