/*====================================================================

ObjectWeb Util Preprocessor Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.cpp.api;

// Package dependencies.
import java.util.Map;
import java.io.File;
import java.io.OutputStream;

/**
 * This defines a simple interface to interact with preprocessors.
 *
 * This provides two preprocess operations.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public interface PreprocessorOperations
{
    /**
     * Preprocesses a file.
     *
     * @param fileName The name of the file to preprocess.
     *
     * @return The file containing the output file.
     */
    public java.io.File
    preprocess(String fileName);

    /**
     * Preprocesses a file.
     *
     * @param fileName The name of the file to preprocess.
     * @param tmpFile The file containing the output file.
     *
     * @return True if OK.
     */
    public boolean
    preprocess(String fileName,
               java.io.File tmpFile);

    /**
     * Preprocesses a file.
     *
     * @param fileName The name of the file to preprocess.
     * @param output The output stream.
     */
    public boolean
    preprocess(String fileName,
               java.io.OutputStream output);
}
