/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.generator.translator.idl2java.lib;

// Package dependencies.

/**
 * This class translates some PSDL declarations to their java mapping.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class PSDL_JavaTranslator
     extends org.objectweb.openccm.generator.translator.idl2java.lib.IDL_JavaTranslator
  implements org.objectweb.openccm.generator.translator.idl2java.api.PSDL_JavaTranslator
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     **/
    public PSDL_JavaTranslator()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.generator.translator.idl2java.api.PSDL_JavaTranslator.
    //
    // ==================================================================

    /**
     * Translate a PsdlTypeRef into its java mapping.
     *
     * @param type - The PsdlTypeRef to translate.
     *
     * @return The resulting mapping.
     **/
    public java.lang.String
    toJava(org.objectweb.openccm.ast.api.PsdlTypeRef type)
    {
        org.objectweb.openccm.ast.api.TypeRef idl_type = null;
        org.objectweb.openccm.ast.api.StorageTypeBase st_base = null;

        if (type == null)
        {
            return "";
        }
        idl_type = type.getTypeRef();
        if (idl_type != null)
        {
            return toJava(idl_type);
        }
        else
        {
            if (type.isRef())
            {   // type is a StorageType reference
                return "byte[]";
            }
            else
            {   // type is a psdl type
                st_base = type.getStorageTypeBase();
                return getAbsoluteName(st_base);
            }
        }
    }

    /**
     * Translate a psdl parameter into its java mapping.
     *
     * @param type - The PsdlTypeRef to translate.
     * @param mode - The parameter mode (in, out or inout).
     *
     * @return The resulting mapping.
     **/
    public java.lang.String
    toJava(org.objectweb.openccm.ast.api.PsdlTypeRef type,
           int mode)
    {
        org.objectweb.openccm.ast.api.TypeRef idl_type = null;
        org.objectweb.openccm.ast.api.StorageTypeBase st_base = null;

        if (type == null)
        {
            return "";
        }
        idl_type = type.getTypeRef();
        if (idl_type != null)
        {
            return toJava(idl_type, mode);
        }
        else
        {
            if (type.isRef())
            {   // type is a StorageType reference
                return "byte[]";
            }
            else
            {   // type is a psdl type
                st_base = type.getStorageTypeBase();
                return getAbsoluteName(st_base)+"Holder";
            }
        }
    }

    /**
     * Translate a psdl reference into its java mapping,
     * i.e. ref<A> => A.
     *
     * @param type - The PsdlTypeRef to translate.
     *
     * @return The resulting mapping.
     **/
    public java.lang.String
    RefToJava(org.objectweb.openccm.ast.api.PsdlTypeRef type)
    {
        org.objectweb.openccm.ast.api.StorageTypeBase st_base = null;

        if (type == null)
        {
            return "";
        }
        st_base = type.getStorageTypeBase();
        return getAbsoluteName(st_base);
    }

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

}
