/*====================================================================

ObjectWeb Util Preprocessor Package.
Copyright (C) 2004 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

--------------------------------------------------------------------
$Id: PreprocessorJPP.java,v 1.1 2004/02/05 20:29:57 rouvoy Exp $
====================================================================*/

package org.objectweb.util.cpp.lib;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Hashtable;
import java.util.Vector;

import org.objectweb.util.misc.api.ExceptionWrapper;


/**
 * This provides a preprocessor written in Java.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @version 0.1
 */
public class PreprocessorJPP
     extends PreprocessorBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public PreprocessorJPP() {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cpp.api.Preprocessor
    //
    // ==================================================================

    /**
     * Preprocesses a file.
     *
     * @param fileName The name of the file to preprocess.
     * @param tmpFile The file containing the output file.
     *
     * @return True if OK.
     */
    public boolean preprocess(String fileName, File tmpFile) {
        FileOutputStream output = null;
        try {
            // Creates an output stream on this temp file.
            output = new FileOutputStream(tmpFile);
        } catch(IOException exc) {
            // Wraps any java.io.IOException exception.
            throw new ExceptionWrapper(exc);
        }

        boolean status = false;
        try {
            // Preprocesses.
            status = preprocess(fileName, output);
        } finally {
            // Always closes the stream. 
            IOHelper.close(output);
        }
        
        return status;
    }

    /**
     * Preprocesses a file.
     *
     * @param fileName The name of the file to preprocess.
     * @param output The output stream.
     */
    public boolean preprocess(String fileName, OutputStream output) {
        try {
            // Creates the preprocessor.
            JPP jpp = new JPP(fileName, output,
                              ((Vector)includeDirectories_.clone()),
                              ((Hashtable)macros_.clone()));
            // Preprocesses.
            return jpp.preprocess();
        } catch(FileNotFoundException exc) {
            getConsole().error(fileName + ": No such file or directory");
            return false;
        } catch(IOException exc) {
            throw new ExceptionWrapper(exc);
        }
    }
}
