/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): .

--------------------------------------------------------------------
$Id: OptionRun.java,v 1.1 2004/02/05 20:29:57 rouvoy Exp $
====================================================================*/

package org.objectweb.util.launcher.option;


import java.util.Vector;

import org.objectweb.util.cmdline.api.Iterator;
import org.objectweb.util.cmdline.lib.DefaultOptionArgument;

import org.objectweb.util.launcher.CommandJava;
import org.objectweb.util.launcher.CommandFactory;

import org.objectweb.util.launcher.parser.Repository;
import org.objectweb.util.launcher.parser.RunDescription;
import org.objectweb.util.trace.TraceSystem;


/**
 * Definition of the --runid option.<BR>
 * <p>
 * This option specify the run identifier to use for starting the 
 * application.
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 * @version 0.1
 */
public class OptionRun
     extends DefaultOptionArgument
  implements OptionLauncher
{
    /** Short Tag Description */
    public final static String shortTag = "-run" ;
    /** Long Tag Description */
    public final static String longTag = "--runid" ;

    /** List of runs to build */
    protected Vector runs_ ;

    /**
     * Default Constructor
     */
    public OptionRun() {
        super(new String[]{shortTag, longTag},
              "<RunID>",
              "The identifier of the run to launch",
              "default");
        runs_ = new Vector();
    }

    /**
     * Loader of run descriptions.
     * 
     * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
     * @version 0.1
     */
    protected class RunBuilder {
        /** Description of the run to use */
        protected String argument_ ;
        
        /**
         * Constructor.
         * @param arg description of the run to build.
         */
        public RunBuilder(String arg) {
            this.argument_ = arg ;
        }
        
        /**
         * Complete the Java Command with the run description.
         * @param cmd the command to complete.
         * @param desc the description of parameters to update.
         */
        protected void completeRun(CommandJava cmd, RunDescription desc) {
            cmd.setName(desc.getName());
            TraceSystem.get("launcher").info("Completing java command " + cmd.getName());
            cmd.setClassname(desc.getMainclass());
            try {
                cmd.getLoader().addURL(desc.getClasses().toStringArray());
            } catch (java.net.MalformedURLException ex) {
                TraceSystem.get("launcher").error("Exception raised: " + ex);
            }
            cmd.addArguments(desc.getArguments());
            cmd.addProperties(desc.getProperties());
        }
        

        /**
         * Complete the command java by setting the identifier of
         * the run to use
         *
         * @param cmd the command to complete
         */
        public CommandJava complete(Repository repository) {
            CommandJava cmd = CommandFactory.instance().create();
            completeRun(cmd,(RunDescription)repository.getDescription(argument_));
            return cmd;
        }
    }
    
    
    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    //
    // ==================================================================
    
    /**
     * Consumes command line arguments from an iterator.
     *
     * @param iterator The command line argument iterator.
     */
    public void consume(Iterator iterator) {
        setArgument(consumeArgument(iterator));
        runs_.addElement(new RunBuilder(getArgument()));
    }
    
    
    
    /**
     * Complete the command java by setting the identifier of
     * the run to use.
     *
     * @param cmd the command to complete.
     */
    public CommandJava[] complete(Repository repository) {
        java.util.Iterator runs = runs_.iterator();
        Vector list = new Vector();
        while (runs.hasNext())
            list.addElement(((RunBuilder)runs.next()).complete(repository));           
        return (CommandJava[])list.toArray(new CommandJava[0]);
    }
       
    /**
     * Creates a commandline representation of the run.
     * @param value value of the run.
     * @return commandline representation of the run.
     */
    public static String create(String run) {
        return shortTag+" "+run+" " ; 
    }
}
