/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2004 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): .

--------------------------------------------------------------------
$Id: TraceTemplate.java,v 1.2 2004/02/27 19:07:21 rouvoy Exp $
====================================================================*/

package org.objectweb.util.trace;


import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;


/**
 * Specialization of the Monolog trace mechanism into a TraceSystem
 * for dealing with trace in the scope of the Launcher.<BR>
 * <p>
 *   The TraceTemplate class provides implementation of the trace 
 *   interface based on Monolog implementation.
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 * @version 0.1
 */
public class TraceTemplate 
  implements Trace
{
    /** underlying Monolog logger reference*/
    protected Logger logger = null;    
    
    /**
     * Default Construtor.
     *
     * @param log Monolog logger to use.
     */
    public TraceTemplate(Logger log) { 
        setLogger(log); 
    }

    /**
     * @return Returns the logger.
     */
    public Logger getLogger() {
        return logger;
    }

    /**
     * @param logger The logger to set.
     */
    public void setLogger(Logger logger) {
        this.logger = logger;
    }

    /**
     * Displays a debug message.
     *
     * @param msg debugging message to print.
     */
    public void debug(String msg) { 
        getLogger().log(BasicLevel.DEBUG, msg); 
    }

    /**
     * Displays a debug message.
     *
     * @param msg information message to print.
     */
    public void info(String msg) { 
        getLogger().log(BasicLevel.INFO, msg); 
    }

    /**
     * Displays a debug message.
     *
     * @param msg warning message to print.
     */
    public void warn(String msg) { 
        getLogger().log(BasicLevel.WARN, msg); 
    }

    /**
     * Displays a debug message.
     *
     * @param msg error message to print.
     */
    public void error(String msg) { 
        getLogger().log(BasicLevel.ERROR, msg); 
    }

    /**
     * Displays a debug message.
     *
     * @param msg fatal message to print.
     */
    public void fatal(String msg) { 
        getLogger().log(BasicLevel.FATAL, msg); 
    }
}
